from __future__ import unicode_literals

from django.core.exceptions import ValidationError as DjangoValidationError
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.reverse import reverse

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.serializers import (
    DocumentSerializer, DocumentTypeSerializer
)

from .models import DocumentMetadata, DocumentTypeMetadataType, MetadataType
from .permissions import permission_document_metadata_add


class MetadataTypeSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        extra_kwargs = {
            'url': {
                'lookup_field': 'pk', 'lookup_url_kwarg': 'metadata_type_pk',
                'view_name': 'rest_api:metadatatype-detail'
            },
        }
        fields = (
            'default', 'id', 'label', 'lookup', 'name', 'parser', 'url',
            'validation'
        )
        model = MetadataType


class DocumentTypeMetadataTypeSerializer(serializers.HyperlinkedModelSerializer):
    document_type = DocumentTypeSerializer(read_only=True)
    metadata_type = MetadataTypeSerializer(read_only=True)
    url = serializers.SerializerMethodField()

    class Meta:
        fields = ('document_type', 'id', 'metadata_type', 'required', 'url')
        model = DocumentTypeMetadataType

    def get_url(self, instance):
        return reverse(
            'rest_api:documenttypemetadatatype-detail', args=(
                instance.document_type.pk, instance.pk
            ), request=self.context['request'], format=self.context['format']
        )


class NewDocumentTypeMetadataTypeSerializer(serializers.ModelSerializer):
    metadata_type_pk = serializers.IntegerField(
        help_text=_('Primary key of the metadata type to be added.'),
        write_only=True
    )
    url = serializers.SerializerMethodField()

    class Meta:
        fields = (
            'id', 'metadata_type_pk', 'required', 'url'
        )
        model = DocumentTypeMetadataType

    def get_url(self, instance):
        return reverse(
            'rest_api:documenttypemetadatatype-detail', args=(
                instance.document_type.pk, instance.pk
            ), request=self.context['request'], format=self.context['format']
        )

    def validate(self, attrs):
        attrs['document_type'] = self.context['document_type']
        attrs['metadata_type'] = MetadataType.objects.get(
            pk=attrs.pop('metadata_type_pk')
        )

        instance = DocumentTypeMetadataType(**attrs)
        try:
            instance.full_clean()
        except DjangoValidationError as exception:
            raise ValidationError(exception)

        return attrs


class WritableDocumentTypeMetadataTypeSerializer(serializers.ModelSerializer):
    url = serializers.SerializerMethodField()

    class Meta:
        fields = (
            'id', 'required', 'url'
        )
        model = DocumentTypeMetadataType

    def get_url(self, instance):
        return reverse(
            'rest_api:documenttypemetadatatype-detail', args=(
                instance.document_type.pk, instance.pk
            ), request=self.context['request'], format=self.context['format']
        )


class DocumentMetadataSerializer(serializers.HyperlinkedModelSerializer):
    document = DocumentSerializer(read_only=True)
    metadata_type = MetadataTypeSerializer(read_only=True)
    url = serializers.SerializerMethodField()

    class Meta:
        fields = ('document', 'id', 'metadata_type', 'url', 'value')
        model = DocumentMetadata
        read_only_fields = ('document', 'metadata_type',)

    def get_url(self, instance):
        return reverse(
            'rest_api:documentmetadata-detail', args=(
                instance.document.pk, instance.pk
            ), request=self.context['request'], format=self.context['format']
        )

    def validate(self, attrs):
        self.instance.value = attrs['value']

        try:
            self.instance.full_clean()
        except DjangoValidationError as exception:
            raise ValidationError(exception)

        return attrs


class NewDocumentMetadataSerializer(serializers.ModelSerializer):
    metadata_type_pk = serializers.IntegerField(
        help_text=_(
            'Primary key of the metadata type to be added to the document.'
        ),
        write_only=True
    )
    url = serializers.SerializerMethodField()

    class Meta:
        fields = ('id', 'metadata_type_pk', 'url', 'value')
        model = DocumentMetadata

    def create(self, validated_data):
        queryset = AccessControlList.objects.restrict_queryset(
            queryset=MetadataType.objects.all(),
            permission=permission_document_metadata_add,
            user=self.context['request'].user
        )
        get_object_or_404(
            klass=queryset, pk=validated_data['metadata_type'].pk
        )

        return super(NewDocumentMetadataSerializer, self).create(
            validated_data=validated_data
        )

    def get_url(self, instance):
        return reverse(
            'rest_api:documentmetadata-detail', args=(
                instance.document.pk, instance.pk
            ), request=self.context['request'], format=self.context['format']
        )

    def validate(self, attrs):
        attrs['document'] = self.context['document']
        attrs['metadata_type'] = MetadataType.objects.get(
            pk=attrs.pop('metadata_type_pk')
        )

        instance = DocumentMetadata(**attrs)
        try:
            instance.full_clean()
        except DjangoValidationError as exception:
            raise ValidationError(exception)

        return attrs



from __future__ import print_function, unicode_literals

import datetime
from errno import ENOENT
import logging
from stat import S_IFDIR, S_IFREG
from time import time

from fuse import FuseOSError, Operations

from django.core.exceptions import MultipleObjectsReturned
from django.db.models import (
    Case, CharField, Count, F, Func, Transform, Value, When
)
from django.db.models.functions import Concat

from mayan.apps.document_indexing.models import Index, IndexInstanceNode
from mayan.apps.documents.models import Document

from .literals import (
    MAX_FILE_DESCRIPTOR, MIN_FILE_DESCRIPTOR, FILE_MODE, DIRECTORY_MODE
)
from .runtime import cache


logger = logging.getLogger(__name__)


class Trim(Transform):
    function = 'TRIM'
    lookup_name = 'trim'


class IndexFilesystem(Operations):
    @staticmethod
    def _clean_queryset(queryset, source_field_name, destination_field_name):
        queryset = IndexFilesystem._clean_queryset_end_of_lines(
            queryset=queryset, source_field_name=source_field_name,
            destination_field_name='_no_newline'
        )

        queryset = IndexFilesystem._clean_queryset_slashes(
            queryset=queryset, source_field_name='_no_newline'
        )

        return IndexFilesystem._clean_queryset_duplicates(
            queryset=queryset, destination_field_name=destination_field_name
        )

    @staticmethod
    def _clean_queryset_end_of_lines(
        queryset, source_field_name='value',
        destination_field_name='clean_value'
    ):
        # Remove newline carriage returns and the first and last space
        # to make multiline indexes valid directoy names
        return queryset.annotate(
            **{
                destination_field_name: Trim(
                    Func(
                        F(source_field_name), Value('\r\n'), Value(' '),
                        function='replace', output_field=CharField()
                    ),
                )
            }
        )

    @staticmethod
    def _clean_queryset_slashes(
        queryset, source_field_name, destination_field_name='_no_slashes'
    ):
        # This is a conditional expression that is executed only for
        # items in the queryset that contain a slash ('/') in their source
        # field. The slash ('/') character is replaced with an
        # underscore ('_').
        return queryset.annotate(
            **{
                destination_field_name: Case(
                    When(
                        **{
                            '{}__contains'.format(source_field_name): '/',
                            'then': Func(
                                F(source_field_name), Value('/'), Value('_'),
                                function='replace', output_field=CharField()
                            )
                        }
                    ),
                    default=source_field_name
                )
            }
        )

    @staticmethod
    def _clean_queryset_duplicates(
        queryset, source_field_name='_no_slashes', destination_field_name='_deduplicated'
    ):
        # Make second queryset of all duplicates
        repeats = queryset.values(source_field_name).annotate(
            repeated_count=Count(source_field_name)
        ).filter(repeated_count__gt=1).values(source_field_name)

        # This is a conditional expression that is executed only for
        # duplicates. The primary key is appended inside a parethesis to
        # the source field.
        return queryset.annotate(
            **{
                destination_field_name: Case(
                    When(
                        **{
                            '{}__in'.format(source_field_name): repeats,
                            'then': Concat(
                                F(source_field_name), Value('('), F('pk'),
                                Value(')'), output_field=CharField()
                            )
                        }
                    ),
                    default=source_field_name
                )
            }
        )

    def _get_next_file_descriptor(self):
        while(True):
            self.file_descriptor_count += 1
            if self.file_descriptor_count > MAX_FILE_DESCRIPTOR:
                self.file_descriptor_count = MIN_FILE_DESCRIPTOR

            try:
                if not self.file_descriptors[self.file_descriptor_count]:
                    return self.file_descriptor_count
            except KeyError:
                return self.file_descriptor_count

    def _path_to_node(self, path, access_only=False, directory_only=True):
        logger.debug('path: %s', path)
        logger.debug('directory_only: %s', directory_only)

        parts = path.split('/')

        logger.debug('parts: %s', parts)

        node = self.index.instance_root

        if len(parts) > 1 and parts[1] != '':
            path_cache = cache.get_path(path=path)

            if path_cache:
                node_pk = path_cache.get('node_pk')
                if node_pk:
                    if access_only:
                        return True
                    else:
                        return IndexInstanceNode.objects.get(pk=node_pk)

                document_pk = path_cache.get('document_pk')
                if document_pk:
                    if access_only:
                        return True
                    else:
                        return Document.objects.get(pk=document_pk)

            for count, part in enumerate(parts[1:]):
                try:
                    node = IndexFilesystem._clean_queryset(
                        queryset=node.get_children(),
                        source_field_name='value',
                        destination_field_name='value_clean'
                    ).get(value_clean=part)
                except IndexInstanceNode.DoesNotExist:
                    logger.debug('%s does not exists', part)

                    if directory_only:
                        return None
                    else:
                        try:
                            if node.index_template_node.link_documents:
                                document = IndexFilesystem._clean_queryset(
                                    queryset=node.documents,
                                    source_field_name='label',
                                    destination_field_name='label_clean'
                                ).get(label_clean=part)

                                logger.debug(
                                    'path %s is a valid file path', path
                                )
                                cache.set_path(path=path, document=document)

                                return document
                            else:
                                return None
                        except Document.DoesNotExist:
                            logger.debug(
                                'path %s is a file, but is not found', path
                            )
                            return None
                        except MultipleObjectsReturned:
                            return None
                except MultipleObjectsReturned:
                    return None

            cache.set_path(path=path, node=node)

        logger.debug('node: %s', node)
        logger.debug('node is root: %s', node.is_root_node())

        return node

    def __init__(self, index_slug):
        self.file_descriptor_count = MIN_FILE_DESCRIPTOR
        self.file_descriptors = {}

        try:
            self.index = Index.objects.get(slug=index_slug)
        except Index.DoesNotExist:
            print('Unknown index slug: {}.'.format(index_slug))
            exit(1)

    def access(self, path, fh=None):
        result = self._path_to_node(
            path=path, access_only=True, directory_only=False
        )

        if not result:
            raise FuseOSError(ENOENT)

    def getattr(self, path, fh=None):
        logger.debug('path: %s, fh: %s', path, fh)

        now = time()
        result = self._path_to_node(path=path, directory_only=False)

        if not result:
            raise FuseOSError(ENOENT)

        # st_nlink tracks the number of hard links to a file.
        # Must be 2 for directories and at least 1 for files
        # https://www.gnu.org/software/libc/manual/html_node/Attribute-Meanings.html
        if isinstance(result, IndexInstanceNode):
            return {
                'st_mode': (S_IFDIR | DIRECTORY_MODE), 'st_ctime': now,
                'st_mtime': now, 'st_atime': now, 'st_nlink': 2
            }
        else:
            return {
                'st_mode': (S_IFREG | FILE_MODE),
                'st_ctime': (
                    result.date_added.replace(tzinfo=None) - result.date_added.utcoffset() - datetime.datetime(1970, 1, 1)
                ).total_seconds(),
                'st_mtime': (
                    result.latest_version.timestamp.replace(tzinfo=None) - result.latest_version.timestamp.utcoffset() - datetime.datetime(1970, 1, 1)
                ).total_seconds(),
                'st_atime': now,
                'st_size': result.size,
                'st_nlink': 1
            }

    def open(self, path, flags):
        result = self._path_to_node(path=path, directory_only=False)

        if isinstance(result, Document):
            next_file_descriptor = self._get_next_file_descriptor()
            self.file_descriptors[next_file_descriptor] = result.open()
            return next_file_descriptor
        else:
            raise FuseOSError(ENOENT)

    def read(self, path, size, offset, fh):
        self.file_descriptors[fh].seek(offset)
        return self.file_descriptors[fh].read(size)

    def readdir(self, path, fh):
        logger.debug('path: %s', path)

        node = self._path_to_node(path=path, directory_only=True)

        if not node:
            raise FuseOSError(ENOENT)

        yield '.'
        yield '..'

        # Index instance nodes to directories
        queryset = IndexFilesystem._clean_queryset(
            queryset=node.get_children(), source_field_name='value',
            destination_field_name='value_clean'
        )

        for value in queryset.values_list('value_clean', flat=True):
            yield value

        # Documents
        if node.index_template_node.link_documents:

            queryset = IndexFilesystem._clean_queryset(
                queryset=node.documents, source_field_name='label',
                destination_field_name='label_clean'
            )

            for value in queryset.values_list('label_clean', flat=True):
                yield value

    def release(self, path, fh):
        self.file_descriptors[fh] = None
        del(self.file_descriptors[fh])



from __future__ import unicode_literals

from django.conf.urls import url

from .api_views import APIKeyListView, APIKeyView
from .views import (
    KeyDeleteView, KeyDetailView, KeyDownloadView, KeyQueryView,
    KeyQueryResultView, KeyReceive, KeyUploadView, PrivateKeyListView,
    PublicKeyListView
)

urlpatterns = [
    url(
        regex=r'^keys/(?P<pk>\d+)/$', view=KeyDetailView.as_view(),
        name='key_detail'
    ),
    url(
        regex=r'^keys/(?P<pk>\d+)/delete/$', view=KeyDeleteView.as_view(),
        name='key_delete'
    ),
    url(
        regex=r'^keys/(?P<pk>\d+)/download/$', view=KeyDownloadView.as_view(),
        name='key_download'
    ),
    url(
        regex=r'^keys/private/$', view=PrivateKeyListView.as_view(),
        name='key_private_list'
    ),
    url(
        regex=r'^keys/public/$', view=PublicKeyListView.as_view(),
        name='key_public_list'
    ),
    url(
        regex=r'^keys/upload/$', view=KeyUploadView.as_view(), name='key_upload'
    ),
    url(regex=r'^keys/query/$', view=KeyQueryView.as_view(), name='key_query'),
    url(
        regex=r'^keys/query/results/$', view=KeyQueryResultView.as_view(),
        name='key_query_results'
    ),
    url(
        regex=r'^keys/receive/(?P<key_id>.+)/$', view=KeyReceive.as_view(),
        name='key_receive'
    ),
]

api_urls = [
    url(
        regex=r'^keys/(?P<pk>[0-9]+)/$', view=APIKeyView.as_view(),
        name='key-detail'
    ),
    url(regex=r'^keys/$', view=APIKeyListView.as_view(), name='key-list'),
]



from __future__ import unicode_literals

from rest_framework import status

from mayan.apps.django_gpg.permissions import permission_key_sign
from mayan.apps.django_gpg.tests.mixins import KeyTestMixin
from mayan.apps.documents.tests.mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..permissions import (
    permission_document_version_sign_detached,
    permission_document_version_sign_embedded,
    permission_document_version_signature_delete,
    permission_document_version_signature_view,
    permission_document_version_signature_upload
)

from .literals import TEST_KEY_PUBLIC_ID, TEST_SIGNED_DOCUMENT_PATH
from .mixins import (
    DetachedSignatureAPIViewTestMixin, EmbeddedSignatureAPIViewTestMixin,
    SignatureTestMixin
)


class DetachedSignatureDocumentAPIViewTestCase(
    DocumentTestMixin, DetachedSignatureAPIViewTestMixin,
    KeyTestMixin, SignatureTestMixin, BaseAPITestCase
):
    auto_upload_document = False

    def test_document_signature_detached_create_view_no_permission(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        response = self._request_test_document_signature_detached_create_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_detached_create_view_with_access(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_signature_upload
        )

        response = self._request_test_document_signature_detached_create_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures + 1
        )

    def test_document_signature_detached_delete_no_permission(self):
        self.upload_document()
        self._upload_test_detached_signature()

        signatures = self.test_document.latest_version.signatures.count()

        response = self._request_test_document_signature_detached_delete_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_detached_delete_with_access(self):
        self.upload_document()
        self._upload_test_detached_signature()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_signature_delete
        )

        response = self._request_test_document_signature_detached_delete_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures - 1
        )

    def test_document_signature_detached_detail_no_permission(self):
        self.upload_document()
        self._upload_test_detached_signature()

        response = self._request_test_document_signature_detached_detail_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_signature_detached_detail_with_access(self):
        self.upload_document()
        self._upload_test_detached_signature()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_signature_view
        )

        response = self._request_test_document_signature_detached_detail_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['key_id'], TEST_KEY_PUBLIC_ID
        )

    def test_document_signature_detached_list_view_no_permission(self):
        self.upload_document()
        self._upload_test_detached_signature()

        response = self._request_test_document_signature_detached_list_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_signature_detached_list_view_with_access(self):
        self.upload_document()
        self._upload_test_detached_signature()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_signature_view
        )

        response = self._request_test_document_signature_detached_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['key_id'], TEST_KEY_PUBLIC_ID
        )

    def test_document_signature_detached_sign_view_with_no_permission(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        response = self._request_test_document_signature_detached_sign_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_detached_sign_view_with_document_access(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_sign_detached
        )

        response = self._request_test_document_signature_detached_sign_view()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_detached_sign_view_with_key_access(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_key_private,
            permission=permission_key_sign
        )

        response = self._request_test_document_signature_detached_sign_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_detached_sign_view_with_full_access(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_sign_detached
        )
        self.grant_access(
            obj=self.test_key_private,
            permission=permission_key_sign
        )

        response = self._request_test_document_signature_detached_sign_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures + 1
        )


class EmbeddedSignatureDocumentAPIViewTestCase(
    DocumentTestMixin, EmbeddedSignatureAPIViewTestMixin,
    KeyTestMixin, BaseAPITestCase
):
    auto_upload_document = False

    def test_document_signature_embedded_sign_view_with_no_permission(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        response = self._request_test_document_signature_embedded_sign_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_embedded_sign_view_with_document_access(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_sign_embedded
        )

        response = self._request_test_document_signature_embedded_sign_view()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_embedded_sign_view_with_key_access(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_key_private,
            permission=permission_key_sign
        )

        response = self._request_test_document_signature_embedded_sign_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures
        )

    def test_document_signature_embedded_sign_view_with_full_access(self):
        self.upload_document()
        self._create_test_key_private()

        signatures = self.test_document.latest_version.signatures.count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_sign_embedded
        )
        self.grant_access(
            obj=self.test_key_private,
            permission=permission_key_sign
        )

        response = self._request_test_document_signature_embedded_sign_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            self.test_document.latest_version.signatures.count(),
            signatures + 1
        )

    def test_document_signature_embedded_detail_no_permission(self):
        self.test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self.upload_document()

        response = self._request_test_document_signature_embedded_detail_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_signature_embedded_detail_with_access(self):
        self.test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self.upload_document()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_signature_view
        )

        response = self._request_test_document_signature_embedded_detail_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['key_id'], TEST_KEY_PUBLIC_ID
        )

    def test_document_signature_embedded_list_view_no_permission(self):
        self.test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self.upload_document()

        response = self._request_test_document_signature_embedded_list_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_document_signature_embedded_list_view_with_access(self):
        self.test_document_path = TEST_SIGNED_DOCUMENT_PATH
        self.upload_document()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_version_signature_view
        )

        response = self._request_test_document_signature_embedded_list_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['key_id'], TEST_KEY_PUBLIC_ID
        )



from __future__ import absolute_import, unicode_literals

from . import *  # NOQA

CELERY_TASK_ALWAYS_EAGER = False

TEMPLATES[0]['OPTIONS']['loaders'] = (  # NOQA: F405
    (
        'django.template.loaders.cached.Loader', (
            'django.template.loaders.filesystem.Loader',
            'django.template.loaders.app_directories.Loader',
        )
    ),
)



from __future__ import unicode_literals

from django.db import transaction

from mayan.apps.user_management.events import event_group_edited

from .events import event_role_edited


def method_group_roles_add(self, queryset, _user):
    with transaction.atomic():
        event_group_edited.commit(
            actor=_user, target=self
        )
        for role in queryset:
            self.roles.add(role)
            event_role_edited.commit(
                actor=_user, action_object=self, target=role
            )


def method_group_roles_remove(self, queryset, _user):
    with transaction.atomic():
        event_group_edited.commit(
            actor=_user, target=self
        )
        for role in queryset:
            self.roles.remove(role)
            event_role_edited.commit(
                actor=_user, action_object=self, target=role
            )



from __future__ import unicode_literals

from importlib import import_module
import itertools
import logging

from django.apps import apps
from django.core.exceptions import PermissionDenied
from django.db.utils import OperationalError, ProgrammingError
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .exceptions import InvalidNamespace

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class PermissionNamespace(object):
    _registry = {}

    @classmethod
    def all(cls):
        return cls._registry.values()

    @classmethod
    def get(cls, name):
        try:
            return cls._registry[name]
        except KeyError:
            raise InvalidNamespace(
                'Invalid namespace name. This is probably an obsolete '
                'permission namespace, execute the management command '
                '"purgepermissions" and try again.'
            )

    def __init__(self, name, label):
        self.name = name
        self.label = label
        self.permissions = []
        self.__class__._registry[name] = self

    def __str__(self):
        return force_text(self.label)

    def add_permission(self, name, label):
        permission = Permission(namespace=self, name=name, label=label)
        self.permissions.append(permission)
        return permission


@python_2_unicode_compatible
class Permission(object):
    _imported_app = []
    _permissions = {}
    _stored_permissions_cache = {}

    @classmethod
    def all(cls, as_choices=False):
        if as_choices:
            results = []

            for namespace, permissions in itertools.groupby(cls.all(), lambda entry: entry.namespace):
                permission_options = [
                    (force_text(permission.pk), permission) for permission in permissions
                ]
                results.append(
                    (namespace, permission_options)
                )

            return results
        else:
            # Return sorted permissions by namespace.name
            return sorted(
                cls._permissions.values(), key=lambda x: x.namespace.name
            )

    @classmethod
    def check_user_permissions(cls, permissions, user):
        for permission in permissions:
            if permission.stored_permission.user_has_this(user=user):
                return True

        logger.debug(
            'User "%s" does not have permissions "%s"', user, permissions
        )
        raise PermissionDenied(_('Insufficient permissions.'))

    @classmethod
    def get(cls, pk, class_only=False):
        if class_only:
            return cls._permissions[pk]
        else:
            return cls._permissions[pk].stored_permission

    @classmethod
    def initialize(cls):
        module_name = 'permissions'

        for app in apps.get_app_configs():
            # Keep track of the apps that have already been imported to
            # avoid importing them more than once. Does not causes a problem,
            # it is an optimization to speed up statups.
            if app not in cls._imported_app:
                try:
                    import_module('{}.{}'.format(app.name, module_name))
                except ImportError as exception:
                    non_fatal_messages = (
                        'No module named {}'.format(module_name),
                        'No module named \'{}.{}\''.format(app.name, module_name)
                    )

                    if force_text(exception) not in non_fatal_messages:
                        logger.error(
                            'Error importing %s %s.py file; %s', app.name,
                            module_name, exception
                        )
                        raise
                finally:
                    cls._imported_app.append(app)

        # Invalidate cache always. This is for tests that build a new memory
        # only database and cause all cache references built in the .ready()
        # method to be invalid.
        cls.invalidate_cache()

        for permission in cls.all():
            permission.stored_permission

    @classmethod
    def invalidate_cache(cls):
        cls._stored_permissions_cache = {}

    def __init__(self, namespace, name, label):
        self.namespace = namespace
        self.name = name
        self.label = label
        self.pk = self.get_pk()
        self.__class__._permissions[self.pk] = self

    def __repr__(self):
        return self.pk

    def __str__(self):
        return force_text(self.label)

    def get_pk(self):
        return '%s.%s' % (self.namespace.name, self.name)

    @property
    def stored_permission(self):
        try:
            return self.__class__._stored_permissions_cache[self.pk]
        except KeyError:
            StoredPermission = apps.get_model(
                app_label='permissions', model_name='StoredPermission'
            )

            try:
                stored_permission, created = StoredPermission.objects.get_or_create(
                    namespace=self.namespace.name,
                    name=self.name,
                )

                self.__class__._stored_permissions_cache[self.pk] = stored_permission
                return stored_permission
            except (OperationalError, ProgrammingError):
                """
                This error is expected when trying to initialize the
                stored permissions during the initial creation of the
                database. Can be safely ignore under that situation.
                """



from __future__ import absolute_import, unicode_literals

from django.contrib import messages
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.urls import reverse, reverse_lazy
from django.utils.html import mark_safe
from django.utils.translation import ugettext_lazy as _, ungettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.common.generics import (
    AddRemoveView, ConfirmView, FormView, SingleObjectCreateView,
    SingleObjectDeleteView, SingleObjectEditView, SingleObjectListView
)
from mayan.apps.documents.events import event_document_type_edited
from mayan.apps.documents.models import Document, DocumentType
from mayan.apps.documents.permissions import (
    permission_document_type_edit, permission_document_view
)
from mayan.apps.documents.views.document_views import DocumentListView

from .events import event_index_template_edited
from .forms import IndexTemplateFilteredForm, IndexTemplateNodeForm
from .html_widgets import node_tree
from .icons import icon_index
from .links import link_index_template_create
from .models import (
    DocumentIndexInstanceNode, Index, IndexInstance, IndexInstanceNode,
    IndexTemplateNode
)
from .permissions import (
    permission_document_indexing_create, permission_document_indexing_delete,
    permission_document_indexing_edit,
    permission_document_indexing_instance_view,
    permission_document_indexing_rebuild, permission_document_indexing_view
)
from .tasks import task_rebuild_index


class DocumentTypeIndexesView(AddRemoveView):
    main_object_permission = permission_document_indexing_edit
    main_object_model = DocumentType
    main_object_pk_url_kwarg = 'pk'
    secondary_object_model = Index
    secondary_object_permission = permission_document_type_edit
    list_available_title = _('Available indexes')
    list_added_title = _('Indexes linked')
    related_field = 'indexes'

    def action_add(self, queryset, _user):
        event_document_type_edited.commit(
            actor=_user, target=self.main_object
        )
        for obj in queryset:
            self.main_object.indexes.add(obj)
            event_index_template_edited.commit(
                actor=_user, action_object=self.main_object, target=obj
            )

    def action_remove(self, queryset, _user):
        event_document_type_edited.commit(
            actor=_user, target=self.main_object
        )
        for obj in queryset:
            self.main_object.indexes.remove(obj)
            event_index_template_edited.commit(
                actor=_user, action_object=self.main_object, target=obj
            )

    def get_actions_extra_kwargs(self):
        return {'_user': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'subtitle': _(
                'Documents of this type will appear in the indexes linked '
                'when these are updated. Events of the documents of this type '
                'will trigger updates in the linked indexes.'
            ),
            'title': _('Indexes linked to document type: %s') % self.main_object,
        }


# Setup views
class SetupIndexCreateView(SingleObjectCreateView):
    extra_context = {'title': _('Create index')}
    fields = ('label', 'slug', 'enabled')
    model = Index
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_setup_list'
    )
    view_permission = permission_document_indexing_create

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class SetupIndexDeleteView(SingleObjectDeleteView):
    model = Index
    object_permission = permission_document_indexing_delete
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_setup_list'
    )

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Delete the index: %s?') % self.get_object(),
        }


class SetupIndexEditView(SingleObjectEditView):
    fields = ('label', 'slug', 'enabled')
    model = Index
    object_permission = permission_document_indexing_edit
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_setup_list'
    )

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Edit index: %s') % self.get_object(),
        }

    def get_save_extra_data(self):
        return {'_user': self.request.user}


class SetupIndexListView(SingleObjectListView):
    model = Index
    object_permission = permission_document_indexing_view

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_index,
            'no_results_main_link': link_index_template_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'Indexes group document automatically into levels. Indexe are '
                'defined using template whose markers are replaced with '
                'direct properties of documents like label or description, or '
                'that of extended properties like metadata.'
            ),
            'no_results_title': _('There are no indexes.'),
            'title': _('Indexes'),
        }


class SetupIndexRebuildView(ConfirmView):
    post_action_redirect = reverse_lazy(
        viewname='indexing:index_setup_list'
    )

    def get_extra_context(self):
        return {
            'object': self.get_object(),
            'title': _('Rebuild index: %s') % self.get_object()
        }

    def get_object(self):
        return get_object_or_404(klass=self.get_queryset(), pk=self.kwargs['pk'])

    def get_queryset(self):
        return AccessControlList.objects.restrict_queryset(
            permission=permission_document_indexing_rebuild,
            queryset=Index.objects.all(), user=self.request.user
        )

    def view_action(self):
        task_rebuild_index.apply_async(
            kwargs=dict(index_id=self.get_object().pk)
        )

        messages.success(
            message=_('Index queued for rebuild.'), request=self.request
        )


class SetupIndexDocumentTypesView(AddRemoveView):
    main_object_method_add = 'document_types_add'
    main_object_method_remove = 'document_types_remove'
    main_object_permission = permission_document_indexing_edit
    main_object_model = Index
    main_object_pk_url_kwarg = 'pk'
    secondary_object_model = DocumentType
    secondary_object_permission = permission_document_type_edit
    list_available_title = _('Available document types')
    list_added_title = _('Document types linked')
    related_field = 'document_types'

    def get_actions_extra_kwargs(self):
        return {'_user': self.request.user}

    def get_extra_context(self):
        return {
            'object': self.main_object,
            'subtitle': _(
                'Only the documents of the types selected will be shown '
                'in the index when built. Only the events of the documents '
                'of the types select will trigger updates in the index.'
            ),
            'title': _('Document types linked to index: %s') % self.main_object,
        }


class SetupIndexTreeTemplateListView(SingleObjectListView):
    object_permission = permission_document_indexing_edit

    def get_extra_context(self):
        return {
            'hide_object': True,
            'index': self.get_index(),
            'navigation_object_list': ('index',),
            'title': _('Tree template nodes for index: %s') % self.get_index(),
        }

    def get_index(self):
        return get_object_or_404(klass=Index, pk=self.kwargs['pk'])

    def get_source_queryset(self):
        return self.get_index().template_root.get_descendants(
            include_self=True
        )


class TemplateNodeCreateView(SingleObjectCreateView):
    form_class = IndexTemplateNodeForm
    model = IndexTemplateNode

    def dispatch(self, request, *args, **kwargs):
        AccessControlList.objects.check_access(
            obj=self.get_parent_node().index,
            permissions=(permission_document_indexing_edit,), user=request.user
        )

        return super(
            TemplateNodeCreateView, self
        ).dispatch(request, *args, **kwargs)

    def get_extra_context(self):
        return {
            'index': self.get_parent_node().index,
            'navigation_object_list': ('index',),
            'title': _('Create child node of: %s') % self.get_parent_node(),
        }

    def get_initial(self):
        parent_node = self.get_parent_node()
        return {
            'index': parent_node.index, 'parent': parent_node
        }

    def get_parent_node(self):
        return get_object_or_404(klass=IndexTemplateNode, pk=self.kwargs['pk'])


class TemplateNodeDeleteView(SingleObjectDeleteView):
    model = IndexTemplateNode
    object_permission = permission_document_indexing_edit

    def get_extra_context(self):
        return {
            'index': self.get_object().index,
            'navigation_object_list': ('index', 'node'),
            'node': self.get_object(),
            'title': _(
                'Delete the index template node: %s?'
            ) % self.get_object(),
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='indexing:index_setup_view', kwargs={
                'pk': self.get_object().index.pk
            }
        )


class TemplateNodeEditView(SingleObjectEditView):
    form_class = IndexTemplateNodeForm
    model = IndexTemplateNode
    object_permission = permission_document_indexing_edit

    def get_extra_context(self):
        return {
            'index': self.get_object().index,
            'navigation_object_list': ('index', 'node'),
            'node': self.get_object(),
            'title': _(
                'Edit the index template node: %s?'
            ) % self.get_object(),
        }

    def get_post_action_redirect(self):
        return reverse(
            viewname='indexing:index_setup_view', kwargs={
                'pk': self.get_object().index.pk
            }
        )


class IndexListView(SingleObjectListView):
    object_permission = permission_document_indexing_instance_view

    def get_extra_context(self):
        return {
            'hide_links': True,
            'hide_object': True,
            'no_results_icon': icon_index,
            'no_results_main_link': link_index_template_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'This could mean that no index templates have been '
                'created or that there index templates '
                'but they are no properly defined.'
            ),
            'no_results_title': _('There are no index instances available.'),
            'title': _('Indexes'),
        }

    def get_source_queryset(self):
        queryset = IndexInstance.objects.filter(enabled=True)
        return queryset.filter(
            node_templates__index_instance_nodes__isnull=False
        ).distinct()


class IndexInstanceNodeView(DocumentListView):
    template_name = 'document_indexing/node_details.html'

    def dispatch(self, request, *args, **kwargs):
        self.index_instance_node = get_object_or_404(
            klass=IndexInstanceNode, pk=self.kwargs['pk']
        )

        AccessControlList.objects.check_access(
            obj=self.index_instance_node.index(),
            permissions=(permission_document_indexing_instance_view,),
            user=request.user
        )

        if self.index_instance_node:
            if self.index_instance_node.index_template_node.link_documents:
                return super(IndexInstanceNodeView, self).dispatch(
                    request, *args, **kwargs
                )

        return SingleObjectListView.dispatch(self, request, *args, **kwargs)

    def get_document_queryset(self):
        if self.index_instance_node:
            if self.index_instance_node.index_template_node.link_documents:
                return self.index_instance_node.documents.all()

    def get_extra_context(self):
        context = super(IndexInstanceNodeView, self).get_extra_context()
        context.update(
            {
                'column_class': 'col-xs-12 col-sm-6 col-md-4 col-lg-3',
                'object': self.index_instance_node,
                'navigation': mark_safe(
                    _('Navigation: %s') % node_tree(
                        node=self.index_instance_node, user=self.request.user
                    )
                ),
                'title': _(
                    'Contents for index: %s'
                ) % self.index_instance_node.get_full_path(),
            }
        )

        if self.index_instance_node and not self.index_instance_node.index_template_node.link_documents:
            context.update(
                {
                    'hide_object': True,
                    'list_as_items': False,
                }
            )

        return context

    def get_source_queryset(self):
        if self.index_instance_node:
            if self.index_instance_node.index_template_node.link_documents:
                return super(IndexInstanceNodeView, self).get_source_queryset()
            else:
                self.object_permission = None
                return self.index_instance_node.get_children().order_by(
                    'value'
                )
        else:
            self.object_permission = None
            return IndexInstanceNode.objects.none()


class DocumentIndexNodeListView(SingleObjectListView):
    """
    Show a list of indexes where the current document can be found
    """
    object_permission = permission_document_indexing_instance_view

    def dispatch(self, request, *args, **kwargs):
        AccessControlList.objects.check_access(
            obj=self.get_document(), permissions=(permission_document_view,),
            user=request.user
        )

        return super(
            DocumentIndexNodeListView, self
        ).dispatch(request, *args, **kwargs)

    def get_document(self):
        return get_object_or_404(klass=Document, pk=self.kwargs['pk'])

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_index,
            'no_results_text': _(
                'Assign the document type of this document '
                'to an index to have it appear in instances of '
                'those indexes organization units. '
            ),
            'no_results_title': _(
                'This document is not in any index'
            ),
            'object': self.get_document(),
            'title': _(
                'Indexes nodes containing document: %s'
            ) % self.get_document(),
        }

    def get_source_queryset(self):
        return DocumentIndexInstanceNode.objects.get_for(
            document=self.get_document()
        )


class IndexesRebuildView(FormView):
    extra_context = {
        'title': _('Rebuild indexes'),
    }
    form_class = IndexTemplateFilteredForm

    def form_valid(self, form):
        count = 0
        for index in form.cleaned_data['index_templates']:
            task_rebuild_index.apply_async(
                kwargs=dict(index_id=index.pk)
            )
            count += 1

        messages.success(
            message=ungettext(
                singular='%(count)d index queued for rebuild.',
                plural='%(count)d indexes queued for rebuild.',
                number=count
            ) % {
                'count': count,
            }, request=self.request
        )

        return super(IndexesRebuildView, self).form_valid(form=form)

    def get_form_extra_kwargs(self):
        return {
            'user': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse(viewname='common:tools_list')


class IndexesResetView(FormView):
    extra_context = {
        'title': _('Reset indexes'),
    }
    form_class = IndexTemplateFilteredForm

    def form_valid(self, form):
        count = 0
        for index in form.cleaned_data['index_templates']:
            index.reset()
            count += 1

        messages.success(
            message=ungettext(
                singular='%(count)d index reset.',
                plural='%(count)d indexes reset.',
                number=count
            ) % {
                'count': count,
            }, request=self.request
        )

        return super(IndexesResetView, self).form_valid(form=form)

    def get_form_extra_kwargs(self):
        return {
            'help_text': _(
                'Index templates for which their instances will be deleted.'
            ),
            'user': self.request.user
        }

    def get_post_action_redirect(self):
        return reverse(viewname='common:tools_list')



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import Namespace

namespace = Namespace(label=_('Document parsing'), name='document_parsing')

setting_auto_parsing = namespace.add_setting(
    global_name='DOCUMENT_PARSING_AUTO_PARSING', default=True,
    help_text=_(
        'Set new document types to perform parsing automatically by default.'
    )
)
setting_pdftotext_path = namespace.add_setting(
    global_name='DOCUMENT_PARSING_PDFTOTEXT_PATH',
    default='/usr/bin/pdftotext',
    help_text=_(
        'File path to poppler\'s pdftotext program used to extract text '
        'from PDF files.'
    ),
    is_path=True
)



from __future__ import unicode_literals

from django.http import HttpResponse
from django.shortcuts import get_object_or_404

from mayan.apps.permissions.classes import Permission
from mayan.apps.rest_api import generics

from .literals import STAGING_FILE_IMAGE_TASK_TIMEOUT
from .models import StagingFolderSource
from .permissions import permission_staging_file_delete
from .serializers import StagingFolderFileSerializer, StagingFolderSerializer
from .storages import storage_staging_file_image_cache
from .tasks import task_generate_staging_file_image


class APIStagingSourceFileView(generics.RetrieveDestroyAPIView):
    """
    get: Details of the selected staging file.
    """
    serializer_class = StagingFolderFileSerializer

    def get_object(self):
        if self.request.method == 'DELETE':
            Permission.check_user_permissions(
                permissions=(permission_staging_file_delete,),
                user=self.request.user
            )

        staging_folder = get_object_or_404(
            klass=StagingFolderSource, pk=self.kwargs['staging_folder_pk']
        )
        return staging_folder.get_file(
            encoded_filename=self.kwargs['encoded_filename']
        )


class APIStagingSourceListView(generics.ListAPIView):
    """
    get: Returns a list of all the staging folders and the files they contain.
    """
    serializer_class = StagingFolderSerializer
    queryset = StagingFolderSource.objects.all()


class APIStagingSourceView(generics.RetrieveAPIView):
    """
    get: Details of the selected staging folders and the files it contains.
    """
    serializer_class = StagingFolderSerializer
    queryset = StagingFolderSource.objects.all()


class APIStagingSourceFileImageView(generics.RetrieveAPIView):
    """
    get: Returns an image representation of the selected document.
    """
    def get_serializer(self, *args, **kwargs):
        return None

    def get_serializer_class(self):
        return None

    def retrieve(self, request, *args, **kwargs):
        width = request.GET.get('width')
        height = request.GET.get('height')

        task = task_generate_staging_file_image.apply_async(
            kwargs=dict(
                staging_folder_pk=self.kwargs['staging_folder_pk'],
                encoded_filename=self.kwargs['encoded_filename'],
                width=width, height=height
            )
        )

        cache_filename = task.get(timeout=STAGING_FILE_IMAGE_TASK_TIMEOUT)

        with storage_staging_file_image_cache.open(cache_filename) as file_object:
            response = HttpResponse(file_object.read(), content_type='image')
            return response



from __future__ import unicode_literals

TEST_CACHE_LABEL = 'test cache label'
TEST_CACHE_MAXIMUM_SIZE = 2 * 2 ** 20  # 2 Megabyte
TEST_CACHE_NAME = 'test_cache_name'
TEST_CACHE_PARTITION_FILE_FILENAME = 'test_cache_partition_file_filename'
TEST_CACHE_PARTITION_FILE_SIZE = 1 * 2 ** 20  # 1 Megabyte
TEST_CACHE_PARTITION_NAME = 'test_cache_partition_name'
TEST_CACHE_STORAGE_INSTANCE_PATH = 'mayan.apps.file_caching.tests.mixins.test_storage'



from __future__ import unicode_literals

from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..models import Comment
from ..permissions import (
    permission_document_comment_create, permission_document_comment_delete,
    permission_document_comment_edit, permission_document_comment_view
)

from .mixins import DocumentCommentTestMixin, DocumentCommentViewTestMixin


class DocumentCommentViewTestCase(
    DocumentCommentViewTestMixin, DocumentCommentTestMixin,
    GenericDocumentViewTestCase
):
    def test_comment_create_view_no_permissions(self):
        comment_count = Comment.objects.count()

        response = self._request_test_comment_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(comment_count, Comment.objects.count())

    def test_comment_create_view_with_permissions(self):
        comment_count = Comment.objects.count()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_comment_create
        )
        response = self._request_test_comment_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(comment_count + 1, Comment.objects.count())

    def test_comment_delete_view_no_permissions(self):
        self._create_test_comment()

        comment_count = Comment.objects.count()

        response = self._request_test_comment_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Comment.objects.count(), comment_count)

    def test_comment_delete_view_with_access(self):
        self._create_test_comment()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_comment_delete
        )

        comment_count = Comment.objects.count()

        response = self._request_test_comment_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Comment.objects.count(), comment_count - 1)

    def test_comment_edit_view_no_permissions(self):
        self._create_test_comment()

        comment_text = self.test_document_comment.comment

        response = self._request_test_comment_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_document_comment.refresh_from_db()
        self.assertEqual(self.test_document_comment.comment, comment_text)

    def test_comment_edit_view_with_access(self):
        self._create_test_comment()

        self.grant_access(
            obj=self.test_document, permission=permission_document_comment_edit
        )

        comment_text = self.test_document_comment.comment

        response = self._request_test_comment_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_document_comment.refresh_from_db()
        self.assertNotEqual(self.test_document_comment.comment, comment_text)

    def test_comment_list_view_with_no_permission(self):
        self._create_test_comment()

        response = self._request_test_comment_list_view()
        self.assertNotContains(
            response=response, text=self.test_document_comment.comment,
            status_code=404
        )

    def test_comment_list_view_with_access(self):
        self._create_test_comment()

        self.grant_access(
            obj=self.test_document,
            permission=permission_document_comment_view
        )

        response = self._request_test_comment_list_view()
        self.assertContains(
            response=response, text=self.test_document_comment.comment,
            status_code=200
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
from django.core.files.storage import FileSystemStorage

import mayan.apps.common.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AnonymousUserSingleton',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
            ],
            options={
                'verbose_name': 'Anonymous user',
                'verbose_name_plural': 'Anonymous user',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='AutoAdminSingleton',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'password', models.CharField(
                        max_length=128, null=True, verbose_name='Password',
                        blank=True
                    )
                ),
                (
                    'password_hash', models.CharField(
                        max_length=128, null=True,
                        verbose_name='Password hash', blank=True
                    )
                ),
                (
                    'account', models.ForeignKey(
                        related_name='auto_admin_account',
                        verbose_name='Account', blank=True,
                        to=settings.AUTH_USER_MODEL, null=True
                    )
                ),
            ],
            options={
                'verbose_name': 'Auto admin properties',
                'verbose_name_plural': 'Auto admin properties',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SharedUploadedFile',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'file', models.FileField(
                        upload_to=mayan.apps.common.models.upload_to,
                        storage=FileSystemStorage(),
                        verbose_name='File'
                    )
                ),
                (
                    'filename', models.CharField(
                        max_length=255, verbose_name='Filename'
                    )
                ),
                (
                    'datatime', models.DateTimeField(
                        auto_now_add=True, verbose_name='Date time'
                    )
                ),
            ],
            options={
                'verbose_name': 'Shared uploaded file',
                'verbose_name_plural': 'Shared uploaded files',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UserLocaleProfile',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'timezone', models.CharField(
                        max_length=48, verbose_name='Timezone', choices=[
                            (b'Africa/Abidjan', b'Africa/Abidjan'),
                            (b'Africa/Accra', b'Africa/Accra'),
                            (b'Africa/Addis_Ababa', b'Africa/Addis_Ababa'),
                            (b'Africa/Algiers', b'Africa/Algiers'),
                            (b'Africa/Asmara', b'Africa/Asmara'),
                            (b'Africa/Bamako', b'Africa/Bamako'),
                            (b'Africa/Bangui', b'Africa/Bangui'),
                            (b'Africa/Banjul', b'Africa/Banjul'),
                            (b'Africa/Bissau', b'Africa/Bissau'),
                            (b'Africa/Blantyre', b'Africa/Blantyre'),
                            (b'Africa/Brazzaville', b'Africa/Brazzaville'),
                            (b'Africa/Bujumbura', b'Africa/Bujumbura'),
                            (b'Africa/Cairo', b'Africa/Cairo'),
                            (b'Africa/Casablanca', b'Africa/Casablanca'),
                            (b'Africa/Ceuta', b'Africa/Ceuta'),
                            (b'Africa/Conakry', b'Africa/Conakry'),
                            (b'Africa/Dakar', b'Africa/Dakar'),
                            (
                                b'Africa/Dar_es_Salaam',
                                b'Africa/Dar_es_Salaam'
                            ),
                            (b'Africa/Djibouti', b'Africa/Djibouti'),
                            (b'Africa/Douala', b'Africa/Douala'),
                            (b'Africa/El_Aaiun', b'Africa/El_Aaiun'),
                            (b'Africa/Freetown', b'Africa/Freetown'),
                            (b'Africa/Gaborone', b'Africa/Gaborone'),
                            (b'Africa/Harare', b'Africa/Harare'),
                            (b'Africa/Johannesburg', b'Africa/Johannesburg'),
                            (b'Africa/Juba', b'Africa/Juba'),
                            (b'Africa/Kampala', b'Africa/Kampala'),
                            (b'Africa/Khartoum', b'Africa/Khartoum'),
                            (b'Africa/Kigali', b'Africa/Kigali'),
                            (b'Africa/Kinshasa', b'Africa/Kinshasa'),
                            (b'Africa/Lagos', b'Africa/Lagos'),
                            (b'Africa/Libreville', b'Africa/Libreville'),
                            (b'Africa/Lome', b'Africa/Lome'),
                            (b'Africa/Luanda', b'Africa/Luanda'),
                            (b'Africa/Lubumbashi', b'Africa/Lubumbashi'),
                            (b'Africa/Lusaka', b'Africa/Lusaka'),
                            (b'Africa/Malabo', b'Africa/Malabo'),
                            (b'Africa/Maputo', b'Africa/Maputo'),
                            (b'Africa/Maseru', b'Africa/Maseru'),
                            (b'Africa/Mbabane', b'Africa/Mbabane'),
                            (b'Africa/Mogadishu', b'Africa/Mogadishu'),
                            (b'Africa/Monrovia', b'Africa/Monrovia'),
                            (b'Africa/Nairobi', b'Africa/Nairobi'),
                            (b'Africa/Ndjamena', b'Africa/Ndjamena'),
                            (b'Africa/Niamey', b'Africa/Niamey'),
                            (b'Africa/Nouakchott', b'Africa/Nouakchott'),
                            (b'Africa/Ouagadougou', b'Africa/Ouagadougou'),
                            (b'Africa/Porto-Novo', b'Africa/Porto-Novo'),
                            (b'Africa/Sao_Tome', b'Africa/Sao_Tome'),
                            (b'Africa/Tripoli', b'Africa/Tripoli'),
                            (b'Africa/Tunis', b'Africa/Tunis'),
                            (b'Africa/Windhoek', b'Africa/Windhoek'),
                            (b'America/Adak', b'America/Adak'),
                            (b'America/Anchorage', b'America/Anchorage'),
                            (b'America/Anguilla', b'America/Anguilla'),
                            (b'America/Antigua', b'America/Antigua'),
                            (b'America/Araguaina', b'America/Araguaina'),
                            (
                                b'America/Argentina/Buenos_Aires',
                                b'America/Argentina/Buenos_Aires'
                            ),
                            (
                                b'America/Argentina/Catamarca',
                                b'America/Argentina/Catamarca'
                            ),
                            (
                                b'America/Argentina/Cordoba',
                                b'America/Argentina/Cordoba'
                            ),
                            (
                                b'America/Argentina/Jujuy',
                                b'America/Argentina/Jujuy'
                            ),
                            (
                                b'America/Argentina/La_Rioja',
                                b'America/Argentina/La_Rioja'
                            ),
                            (
                                b'America/Argentina/Mendoza',
                                b'America/Argentina/Mendoza'
                            ),
                            (
                                b'America/Argentina/Rio_Gallegos',
                                b'America/Argentina/Rio_Gallegos'
                            ),
                            (
                                b'America/Argentina/Salta',
                                b'America/Argentina/Salta'
                            ),
                            (
                                b'America/Argentina/San_Juan',
                                b'America/Argentina/San_Juan'
                            ),
                            (
                                b'America/Argentina/San_Luis',
                                b'America/Argentina/San_Luis'
                            ),
                            (
                                b'America/Argentina/Tucuman',
                                b'America/Argentina/Tucuman'
                            ),
                            (
                                b'America/Argentina/Ushuaia',
                                b'America/Argentina/Ushuaia'
                            ),
                            (b'America/Aruba', b'America/Aruba'),
                            (b'America/Asuncion', b'America/Asuncion'),
                            (b'America/Atikokan', b'America/Atikokan'),
                            (b'America/Bahia', b'America/Bahia'),
                            (
                                b'America/Bahia_Banderas',
                                b'America/Bahia_Banderas'
                            ),
                            (b'America/Barbados', b'America/Barbados'),
                            (b'America/Belem', b'America/Belem'),
                            (b'America/Belize', b'America/Belize'),
                            (b'America/Blanc-Sablon', b'America/Blanc-Sablon'),
                            (b'America/Boa_Vista', b'America/Boa_Vista'),
                            (b'America/Bogota', b'America/Bogota'),
                            (b'America/Boise', b'America/Boise'),
                            (
                                b'America/Cambridge_Bay',
                                b'America/Cambridge_Bay'
                            ),
                            (b'America/Campo_Grande', b'America/Campo_Grande'),
                            (b'America/Cancun', b'America/Cancun'),
                            (b'America/Caracas', b'America/Caracas'),
                            (b'America/Cayenne', b'America/Cayenne'),
                            (b'America/Cayman', b'America/Cayman'),
                            (b'America/Chicago', b'America/Chicago'),
                            (b'America/Chihuahua', b'America/Chihuahua'),
                            (b'America/Costa_Rica', b'America/Costa_Rica'),
                            (b'America/Creston', b'America/Creston'),
                            (b'America/Cuiaba', b'America/Cuiaba'),
                            (b'America/Curacao', b'America/Curacao'),
                            (b'America/Danmarkshavn', b'America/Danmarkshavn'),
                            (b'America/Dawson', b'America/Dawson'),
                            (b'America/Dawson_Creek', b'America/Dawson_Creek'),
                            (b'America/Denver', b'America/Denver'),
                            (b'America/Detroit', b'America/Detroit'),
                            (b'America/Dominica', b'America/Dominica'),
                            (b'America/Edmonton', b'America/Edmonton'),
                            (b'America/Eirunepe', b'America/Eirunepe'),
                            (b'America/El_Salvador', b'America/El_Salvador'),
                            (b'America/Fortaleza', b'America/Fortaleza'),
                            (b'America/Glace_Bay', b'America/Glace_Bay'),
                            (b'America/Godthab', b'America/Godthab'),
                            (b'America/Goose_Bay', b'America/Goose_Bay'),
                            (b'America/Grand_Turk', b'America/Grand_Turk'),
                            (b'America/Grenada', b'America/Grenada'),
                            (b'America/Guadeloupe', b'America/Guadeloupe'),
                            (b'America/Guatemala', b'America/Guatemala'),
                            (b'America/Guayaquil', b'America/Guayaquil'),
                            (b'America/Guyana', b'America/Guyana'),
                            (b'America/Halifax', b'America/Halifax'),
                            (b'America/Havana', b'America/Havana'),
                            (b'America/Hermosillo', b'America/Hermosillo'),
                            (
                                b'America/Indiana/Indianapolis',
                                b'America/Indiana/Indianapolis'
                            ),
                            (b'America/Indiana/Knox', b'America/Indiana/Knox'),
                            (
                                b'America/Indiana/Marengo',
                                b'America/Indiana/Marengo'
                            ),
                            (
                                b'America/Indiana/Petersburg',
                                b'America/Indiana/Petersburg'
                            ),
                            (
                                b'America/Indiana/Tell_City',
                                b'America/Indiana/Tell_City'
                            ),
                            (
                                b'America/Indiana/Vevay',
                                b'America/Indiana/Vevay'
                            ),
                            (
                                b'America/Indiana/Vincennes',
                                b'America/Indiana/Vincennes'
                            ),
                            (
                                b'America/Indiana/Winamac',
                                b'America/Indiana/Winamac'
                            ),
                            (b'America/Inuvik', b'America/Inuvik'),
                            (b'America/Iqaluit', b'America/Iqaluit'),
                            (b'America/Jamaica', b'America/Jamaica'),
                            (b'America/Juneau', b'America/Juneau'),
                            (
                                b'America/Kentucky/Louisville',
                                b'America/Kentucky/Louisville'
                            ),
                            (
                                b'America/Kentucky/Monticello',
                                b'America/Kentucky/Monticello'
                            ),
                            (b'America/Kralendijk', b'America/Kralendijk'),
                            (b'America/La_Paz', b'America/La_Paz'),
                            (b'America/Lima', b'America/Lima'),
                            (b'America/Los_Angeles', b'America/Los_Angeles'),
                            (
                                b'America/Lower_Princes',
                                b'America/Lower_Princes'
                            ),
                            (b'America/Maceio', b'America/Maceio'),
                            (b'America/Managua', b'America/Managua'),
                            (b'America/Manaus', b'America/Manaus'),
                            (b'America/Marigot', b'America/Marigot'),
                            (b'America/Martinique', b'America/Martinique'),
                            (b'America/Matamoros', b'America/Matamoros'),
                            (b'America/Mazatlan', b'America/Mazatlan'),
                            (b'America/Menominee', b'America/Menominee'),
                            (b'America/Merida', b'America/Merida'),
                            (b'America/Metlakatla', b'America/Metlakatla'),
                            (b'America/Mexico_City', b'America/Mexico_City'),
                            (b'America/Miquelon', b'America/Miquelon'),
                            (b'America/Moncton', b'America/Moncton'),
                            (b'America/Monterrey', b'America/Monterrey'),
                            (b'America/Montevideo', b'America/Montevideo'),
                            (b'America/Montreal', b'America/Montreal'),
                            (b'America/Montserrat', b'America/Montserrat'),
                            (b'America/Nassau', b'America/Nassau'),
                            (b'America/New_York', b'America/New_York'),
                            (b'America/Nipigon', b'America/Nipigon'),
                            (b'America/Nome', b'America/Nome'),
                            (b'America/Noronha', b'America/Noronha'),
                            (
                                b'America/North_Dakota/Beulah',
                                b'America/North_Dakota/Beulah'
                            ),
                            (
                                b'America/North_Dakota/Center',
                                b'America/North_Dakota/Center'),
                            (
                                b'America/North_Dakota/New_Salem',
                                b'America/North_Dakota/New_Salem'),
                            (b'America/Ojinaga', b'America/Ojinaga'),
                            (b'America/Panama', b'America/Panama'),
                            (b'America/Pangnirtung', b'America/Pangnirtung'),
                            (b'America/Paramaribo', b'America/Paramaribo'),
                            (b'America/Phoenix', b'America/Phoenix'),
                            (
                                b'America/Port-au-Prince',
                                b'America/Port-au-Prince'
                            ),
                            (
                                b'America/Port_of_Spain',
                                b'America/Port_of_Spain'
                            ),
                            (b'America/Porto_Velho', b'America/Porto_Velho'),
                            (b'America/Puerto_Rico', b'America/Puerto_Rico'),
                            (b'America/Rainy_River', b'America/Rainy_River'),
                            (b'America/Rankin_Inlet', b'America/Rankin_Inlet'),
                            (b'America/Recife', b'America/Recife'),
                            (b'America/Regina', b'America/Regina'),
                            (b'America/Resolute', b'America/Resolute'),
                            (b'America/Rio_Branco', b'America/Rio_Branco'),
                            (b'America/Santa_Isabel', b'America/Santa_Isabel'),
                            (b'America/Santarem', b'America/Santarem'),
                            (b'America/Santiago', b'America/Santiago'),
                            (
                                b'America/Santo_Domingo',
                                b'America/Santo_Domingo'
                            ),
                            (b'America/Sao_Paulo', b'America/Sao_Paulo'),
                            (b'America/Scoresbysund', b'America/Scoresbysund'),
                            (b'America/Sitka', b'America/Sitka'),
                            (
                                b'America/St_Barthelemy',
                                b'America/St_Barthelemy'
                            ),
                            (b'America/St_Johns', b'America/St_Johns'),
                            (b'America/St_Kitts', b'America/St_Kitts'),
                            (b'America/St_Lucia', b'America/St_Lucia'),
                            (b'America/St_Thomas', b'America/St_Thomas'),
                            (b'America/St_Vincent', b'America/St_Vincent'),
                            (
                                b'America/Swift_Current',
                                b'America/Swift_Current'
                            ),
                            (b'America/Tegucigalpa', b'America/Tegucigalpa'),
                            (b'America/Thule', b'America/Thule'),
                            (b'America/Thunder_Bay', b'America/Thunder_Bay'),
                            (b'America/Tijuana', b'America/Tijuana'),
                            (b'America/Toronto', b'America/Toronto'),
                            (b'America/Tortola', b'America/Tortola'),
                            (b'America/Vancouver', b'America/Vancouver'),
                            (b'America/Whitehorse', b'America/Whitehorse'),
                            (b'America/Winnipeg', b'America/Winnipeg'),
                            (b'America/Yakutat', b'America/Yakutat'),
                            (b'America/Yellowknife', b'America/Yellowknife'),
                            (b'Antarctica/Casey', b'Antarctica/Casey'),
                            (b'Antarctica/Davis', b'Antarctica/Davis'),
                            (
                                b'Antarctica/DumontDUrville',
                                b'Antarctica/DumontDUrville'
                            ),
                            (b'Antarctica/Macquarie', b'Antarctica/Macquarie'),
                            (b'Antarctica/Mawson', b'Antarctica/Mawson'),
                            (b'Antarctica/McMurdo', b'Antarctica/McMurdo'),
                            (b'Antarctica/Palmer', b'Antarctica/Palmer'),
                            (b'Antarctica/Rothera', b'Antarctica/Rothera'),
                            (b'Antarctica/Syowa', b'Antarctica/Syowa'),
                            (b'Antarctica/Troll', b'Antarctica/Troll'),
                            (b'Antarctica/Vostok', b'Antarctica/Vostok'),
                            (b'Arctic/Longyearbyen', b'Arctic/Longyearbyen'),
                            (b'Asia/Aden', b'Asia/Aden'),
                            (b'Asia/Almaty', b'Asia/Almaty'),
                            (b'Asia/Amman', b'Asia/Amman'),
                            (b'Asia/Anadyr', b'Asia/Anadyr'),
                            (b'Asia/Aqtau', b'Asia/Aqtau'),
                            (b'Asia/Aqtobe', b'Asia/Aqtobe'),
                            (b'Asia/Ashgabat', b'Asia/Ashgabat'),
                            (b'Asia/Baghdad', b'Asia/Baghdad'),
                            (b'Asia/Bahrain', b'Asia/Bahrain'),
                            (b'Asia/Baku', b'Asia/Baku'),
                            (b'Asia/Bangkok', b'Asia/Bangkok'),
                            (b'Asia/Beirut', b'Asia/Beirut'),
                            (b'Asia/Bishkek', b'Asia/Bishkek'),
                            (b'Asia/Brunei', b'Asia/Brunei'),
                            (b'Asia/Choibalsan', b'Asia/Choibalsan'),
                            (b'Asia/Chongqing', b'Asia/Chongqing'),
                            (b'Asia/Colombo', b'Asia/Colombo'),
                            (b'Asia/Damascus', b'Asia/Damascus'),
                            (b'Asia/Dhaka', b'Asia/Dhaka'),
                            (b'Asia/Dili', b'Asia/Dili'),
                            (b'Asia/Dubai', b'Asia/Dubai'),
                            (b'Asia/Dushanbe', b'Asia/Dushanbe'),
                            (b'Asia/Gaza', b'Asia/Gaza'),
                            (b'Asia/Harbin', b'Asia/Harbin'),
                            (b'Asia/Hebron', b'Asia/Hebron'),
                            (b'Asia/Ho_Chi_Minh', b'Asia/Ho_Chi_Minh'),
                            (b'Asia/Hong_Kong', b'Asia/Hong_Kong'),
                            (b'Asia/Hovd', b'Asia/Hovd'),
                            (b'Asia/Irkutsk', b'Asia/Irkutsk'),
                            (b'Asia/Jakarta', b'Asia/Jakarta'),
                            (b'Asia/Jayapura', b'Asia/Jayapura'),
                            (b'Asia/Jerusalem', b'Asia/Jerusalem'),
                            (b'Asia/Kabul', b'Asia/Kabul'),
                            (b'Asia/Kamchatka', b'Asia/Kamchatka'),
                            (b'Asia/Karachi', b'Asia/Karachi'),
                            (b'Asia/Kashgar', b'Asia/Kashgar'),
                            (b'Asia/Kathmandu', b'Asia/Kathmandu'),
                            (b'Asia/Khandyga', b'Asia/Khandyga'),
                            (b'Asia/Kolkata', b'Asia/Kolkata'),
                            (b'Asia/Krasnoyarsk', b'Asia/Krasnoyarsk'),
                            (b'Asia/Kuala_Lumpur', b'Asia/Kuala_Lumpur'),
                            (b'Asia/Kuching', b'Asia/Kuching'),
                            (b'Asia/Kuwait', b'Asia/Kuwait'),
                            (b'Asia/Macau', b'Asia/Macau'),
                            (b'Asia/Magadan', b'Asia/Magadan'),
                            (b'Asia/Makassar', b'Asia/Makassar'),
                            (b'Asia/Manila', b'Asia/Manila'),
                            (b'Asia/Muscat', b'Asia/Muscat'),
                            (b'Asia/Nicosia', b'Asia/Nicosia'),
                            (b'Asia/Novokuznetsk', b'Asia/Novokuznetsk'),
                            (b'Asia/Novosibirsk', b'Asia/Novosibirsk'),
                            (b'Asia/Omsk', b'Asia/Omsk'),
                            (b'Asia/Oral', b'Asia/Oral'),
                            (b'Asia/Phnom_Penh', b'Asia/Phnom_Penh'),
                            (b'Asia/Pontianak', b'Asia/Pontianak'),
                            (b'Asia/Pyongyang', b'Asia/Pyongyang'),
                            (b'Asia/Qatar', b'Asia/Qatar'),
                            (b'Asia/Qyzylorda', b'Asia/Qyzylorda'),
                            (b'Asia/Rangoon', b'Asia/Rangoon'),
                            (b'Asia/Riyadh', b'Asia/Riyadh'),
                            (b'Asia/Sakhalin', b'Asia/Sakhalin'),
                            (b'Asia/Samarkand', b'Asia/Samarkand'),
                            (b'Asia/Seoul', b'Asia/Seoul'),
                            (b'Asia/Shanghai', b'Asia/Shanghai'),
                            (b'Asia/Singapore', b'Asia/Singapore'),
                            (b'Asia/Taipei', b'Asia/Taipei'),
                            (b'Asia/Tashkent', b'Asia/Tashkent'),
                            (b'Asia/Tbilisi', b'Asia/Tbilisi'),
                            (b'Asia/Tehran', b'Asia/Tehran'),
                            (b'Asia/Thimphu', b'Asia/Thimphu'),
                            (b'Asia/Tokyo', b'Asia/Tokyo'),
                            (b'Asia/Ulaanbaatar', b'Asia/Ulaanbaatar'),
                            (b'Asia/Urumqi', b'Asia/Urumqi'),
                            (b'Asia/Ust-Nera', b'Asia/Ust-Nera'),
                            (b'Asia/Vientiane', b'Asia/Vientiane'),
                            (b'Asia/Vladivostok', b'Asia/Vladivostok'),
                            (b'Asia/Yakutsk', b'Asia/Yakutsk'),
                            (b'Asia/Yekaterinburg', b'Asia/Yekaterinburg'),
                            (b'Asia/Yerevan', b'Asia/Yerevan'),
                            (b'Atlantic/Azores', b'Atlantic/Azores'),
                            (b'Atlantic/Bermuda', b'Atlantic/Bermuda'),
                            (b'Atlantic/Canary', b'Atlantic/Canary'),
                            (b'Atlantic/Cape_Verde', b'Atlantic/Cape_Verde'),
                            (b'Atlantic/Faroe', b'Atlantic/Faroe'),
                            (b'Atlantic/Madeira', b'Atlantic/Madeira'),
                            (b'Atlantic/Reykjavik', b'Atlantic/Reykjavik'),
                            (
                                b'Atlantic/South_Georgia',
                                b'Atlantic/South_Georgia'
                            ),
                            (b'Atlantic/St_Helena', b'Atlantic/St_Helena'),
                            (b'Atlantic/Stanley', b'Atlantic/Stanley'),
                            (b'Australia/Adelaide', b'Australia/Adelaide'),
                            (b'Australia/Brisbane', b'Australia/Brisbane'),
                            (
                                b'Australia/Broken_Hill',
                                b'Australia/Broken_Hill'
                            ),
                            (b'Australia/Currie', b'Australia/Currie'),
                            (b'Australia/Darwin', b'Australia/Darwin'),
                            (b'Australia/Eucla', b'Australia/Eucla'),
                            (b'Australia/Hobart', b'Australia/Hobart'),
                            (b'Australia/Lindeman', b'Australia/Lindeman'),
                            (b'Australia/Lord_Howe', b'Australia/Lord_Howe'),
                            (b'Australia/Melbourne', b'Australia/Melbourne'),
                            (b'Australia/Perth', b'Australia/Perth'),
                            (b'Australia/Sydney', b'Australia/Sydney'),
                            (b'Canada/Atlantic', b'Canada/Atlantic'),
                            (b'Canada/Central', b'Canada/Central'),
                            (b'Canada/Eastern', b'Canada/Eastern'),
                            (b'Canada/Mountain', b'Canada/Mountain'),
                            (b'Canada/Newfoundland', b'Canada/Newfoundland'),
                            (b'Canada/Pacific', b'Canada/Pacific'),
                            (b'Europe/Amsterdam', b'Europe/Amsterdam'),
                            (b'Europe/Andorra', b'Europe/Andorra'),
                            (b'Europe/Athens', b'Europe/Athens'),
                            (b'Europe/Belgrade', b'Europe/Belgrade'),
                            (b'Europe/Berlin', b'Europe/Berlin'),
                            (b'Europe/Bratislava', b'Europe/Bratislava'),
                            (b'Europe/Brussels', b'Europe/Brussels'),
                            (b'Europe/Bucharest', b'Europe/Bucharest'),
                            (b'Europe/Budapest', b'Europe/Budapest'),
                            (b'Europe/Busingen', b'Europe/Busingen'),
                            (b'Europe/Chisinau', b'Europe/Chisinau'),
                            (b'Europe/Copenhagen', b'Europe/Copenhagen'),
                            (b'Europe/Dublin', b'Europe/Dublin'),
                            (b'Europe/Gibraltar', b'Europe/Gibraltar'),
                            (b'Europe/Guernsey', b'Europe/Guernsey'),
                            (b'Europe/Helsinki', b'Europe/Helsinki'),
                            (b'Europe/Isle_of_Man', b'Europe/Isle_of_Man'),
                            (b'Europe/Istanbul', b'Europe/Istanbul'),
                            (b'Europe/Jersey', b'Europe/Jersey'),
                            (b'Europe/Kaliningrad', b'Europe/Kaliningrad'),
                            (b'Europe/Kiev', b'Europe/Kiev'),
                            (b'Europe/Lisbon', b'Europe/Lisbon'),
                            (b'Europe/Ljubljana', b'Europe/Ljubljana'),
                            (b'Europe/London', b'Europe/London'),
                            (b'Europe/Luxembourg', b'Europe/Luxembourg'),
                            (b'Europe/Madrid', b'Europe/Madrid'),
                            (b'Europe/Malta', b'Europe/Malta'),
                            (b'Europe/Mariehamn', b'Europe/Mariehamn'),
                            (b'Europe/Minsk', b'Europe/Minsk'),
                            (b'Europe/Monaco', b'Europe/Monaco'),
                            (b'Europe/Moscow', b'Europe/Moscow'),
                            (b'Europe/Oslo', b'Europe/Oslo'),
                            (b'Europe/Paris', b'Europe/Paris'),
                            (b'Europe/Podgorica', b'Europe/Podgorica'),
                            (b'Europe/Prague', b'Europe/Prague'),
                            (b'Europe/Riga', b'Europe/Riga'),
                            (b'Europe/Rome', b'Europe/Rome'),
                            (b'Europe/Samara', b'Europe/Samara'),
                            (b'Europe/San_Marino', b'Europe/San_Marino'),
                            (b'Europe/Sarajevo', b'Europe/Sarajevo'),
                            (b'Europe/Simferopol', b'Europe/Simferopol'),
                            (b'Europe/Skopje', b'Europe/Skopje'),
                            (b'Europe/Sofia', b'Europe/Sofia'),
                            (b'Europe/Stockholm', b'Europe/Stockholm'),
                            (b'Europe/Tallinn', b'Europe/Tallinn'),
                            (b'Europe/Tirane', b'Europe/Tirane'),
                            (b'Europe/Uzhgorod', b'Europe/Uzhgorod'),
                            (b'Europe/Vaduz', b'Europe/Vaduz'),
                            (b'Europe/Vatican', b'Europe/Vatican'),
                            (b'Europe/Vienna', b'Europe/Vienna'),
                            (b'Europe/Vilnius', b'Europe/Vilnius'),
                            (b'Europe/Volgograd', b'Europe/Volgograd'),
                            (b'Europe/Warsaw', b'Europe/Warsaw'),
                            (b'Europe/Zagreb', b'Europe/Zagreb'),
                            (b'Europe/Zaporozhye', b'Europe/Zaporozhye'),
                            (b'Europe/Zurich', b'Europe/Zurich'),
                            (b'GMT', b'GMT'),
                            (b'Indian/Antananarivo', b'Indian/Antananarivo'),
                            (b'Indian/Chagos', b'Indian/Chagos'),
                            (b'Indian/Christmas', b'Indian/Christmas'),
                            (b'Indian/Cocos', b'Indian/Cocos'),
                            (b'Indian/Comoro', b'Indian/Comoro'),
                            (b'Indian/Kerguelen', b'Indian/Kerguelen'),
                            (b'Indian/Mahe', b'Indian/Mahe'),
                            (b'Indian/Maldives', b'Indian/Maldives'),
                            (b'Indian/Mauritius', b'Indian/Mauritius'),
                            (b'Indian/Mayotte', b'Indian/Mayotte'),
                            (b'Indian/Reunion', b'Indian/Reunion'),
                            (b'Pacific/Apia', b'Pacific/Apia'),
                            (b'Pacific/Auckland', b'Pacific/Auckland'),
                            (b'Pacific/Chatham', b'Pacific/Chatham'),
                            (b'Pacific/Chuuk', b'Pacific/Chuuk'),
                            (b'Pacific/Easter', b'Pacific/Easter'),
                            (b'Pacific/Efate', b'Pacific/Efate'),
                            (b'Pacific/Enderbury', b'Pacific/Enderbury'),
                            (b'Pacific/Fakaofo', b'Pacific/Fakaofo'),
                            (b'Pacific/Fiji', b'Pacific/Fiji'),
                            (b'Pacific/Funafuti', b'Pacific/Funafuti'),
                            (b'Pacific/Galapagos', b'Pacific/Galapagos'),
                            (b'Pacific/Gambier', b'Pacific/Gambier'),
                            (b'Pacific/Guadalcanal', b'Pacific/Guadalcanal'),
                            (b'Pacific/Guam', b'Pacific/Guam'),
                            (b'Pacific/Honolulu', b'Pacific/Honolulu'),
                            (b'Pacific/Johnston', b'Pacific/Johnston'),
                            (b'Pacific/Kiritimati', b'Pacific/Kiritimati'),
                            (b'Pacific/Kosrae', b'Pacific/Kosrae'),
                            (b'Pacific/Kwajalein', b'Pacific/Kwajalein'),
                            (b'Pacific/Majuro', b'Pacific/Majuro'),
                            (b'Pacific/Marquesas', b'Pacific/Marquesas'),
                            (b'Pacific/Midway', b'Pacific/Midway'),
                            (b'Pacific/Nauru', b'Pacific/Nauru'),
                            (b'Pacific/Niue', b'Pacific/Niue'),
                            (b'Pacific/Norfolk', b'Pacific/Norfolk'),
                            (b'Pacific/Noumea', b'Pacific/Noumea'),
                            (b'Pacific/Pago_Pago', b'Pacific/Pago_Pago'),
                            (b'Pacific/Palau', b'Pacific/Palau'),
                            (b'Pacific/Pitcairn', b'Pacific/Pitcairn'),
                            (b'Pacific/Pohnpei', b'Pacific/Pohnpei'),
                            (b'Pacific/Port_Moresby', b'Pacific/Port_Moresby'),
                            (b'Pacific/Rarotonga', b'Pacific/Rarotonga'),
                            (b'Pacific/Saipan', b'Pacific/Saipan'),
                            (b'Pacific/Tahiti', b'Pacific/Tahiti'),
                            (b'Pacific/Tarawa', b'Pacific/Tarawa'),
                            (b'Pacific/Tongatapu', b'Pacific/Tongatapu'),
                            (b'Pacific/Wake', b'Pacific/Wake'),
                            (b'Pacific/Wallis', b'Pacific/Wallis'),
                            (b'US/Alaska', b'US/Alaska'),
                            (b'US/Arizona', b'US/Arizona'),
                            (b'US/Central', b'US/Central'),
                            (b'US/Eastern', b'US/Eastern'),
                            (b'US/Hawaii', b'US/Hawaii'),
                            (b'US/Mountain', b'US/Mountain'),
                            (b'US/Pacific', b'US/Pacific'),
                            (b'UTC', b'UTC')
                        ]
                    )
                ),
                (
                    'language', models.CharField(
                        max_length=8, verbose_name='Language', choices=[
                            ('ar', 'Arabic'), ('bg', 'Bulgarian'),
                            ('bs', 'Bosnian (Bosnia and Herzegovina)'),
                            ('da', 'Danish'), ('de', 'German (Germany)'),
                            ('en', 'English'), ('es', 'Spanish'),
                            ('fa', 'Persian'), ('fr', 'French'),
                            ('hu', 'Hungarian'), ('hr', 'Croatian'),
                            ('id', 'Indonesian'), ('it', 'Italian'),
                            ('nl', 'Dutch (Nethherlands)'), ('pl', 'Polish'),
                            ('pt', 'Portuguese'),
                            ('pt-br', 'Portuguese (Brazil)'),
                            ('ro', 'Romanian (Romania)'), ('ru', 'Russian'),
                            ('sl', 'Slovenian'), ('tr', 'Turkish'),
                            ('vi', 'Vietnamese (Viet Nam)'),
                            ('zh-cn', 'Chinese (China)')
                        ]
                    )
                ),
                (
                    'user', models.OneToOneField(
                        related_name='locale_profile', verbose_name='User',
                        to=settings.AUTH_USER_MODEL
                    )
                ),
            ],
            options={
                'verbose_name': 'User locale profile',
                'verbose_name_plural': 'User locale profiles',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals

from django.core import management

# '*.html*', removed. It is required by drf-yasg/ to serve oauth2-redirect.html
IGNORE_LIST = [
    'AUTHORS*', 'CHANGE*', 'CONTRIBUT*', 'CODE_OF_CONDUCT*', 'Grunt*',
    'LICENSE*', 'MAINTAIN*', 'README*', '*.less', '*.md', '*.nupkg',
    '*.nuspec', '*.scss*', '*.sh', '*tests*', 'bower*', 'composer.json*',
    'demo*', 'docs', 'grunt*', 'gulp*', 'install', 'less', 'package.json*',
    'package-lock*', 'test', 'tests', 'variable*',
]


class Command(management.BaseCommand):
    help = 'Call the collectstatic command with some specific defaults.'

    def add_arguments(self, parser):
        parser.add_argument(
            '--noinput', '--no-input',
            action='store_false', dest='interactive', default=True,
            help='Do NOT prompt the user for input of any kind.',
        )
        parser.add_argument(
            '-n', '--dry-run',
            action='store_true', dest='dry_run', default=False,
            help='Do everything except modify the filesystem.',
        )
        parser.add_argument(
            '-c', '--clear',
            action='store_true', dest='clear', default=False,
            help='Clear the existing files using the storage '
                 'before trying to copy or link the original file.',
        )
        parser.add_argument(
            '-l', '--link',
            action='store_true', dest='link', default=False,
            help='Create a symbolic link to each file instead of copying.',
        )

    def handle(self, *app_labels, **options):
        management.call_command(
            command_name='collectstatic',
            clear=options['clear'],
            dry_run=options['dry_run'],
            ignore=IGNORE_LIST,
            interactive=options['interactive'],
            link=options['link'],
        )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Access control lists'), name='acls')

permission_acl_edit = namespace.add_permission(
    label=_('Edit ACLs'), name='acl_edit'
)
permission_acl_view = namespace.add_permission(
    label=_('View ACLs'), name='acl_view'
)



from __future__ import unicode_literals

TEST_WEB_LINK_LABEL = 'test web link label'
TEST_WEB_LINK_LABEL_EDITED = 'test web link label edited'
TEST_WEB_LINK_TEMPLATE = 'http://www.example.com/document-{{ document.uuid }}'



from __future__ import absolute_import, unicode_literals

from django.shortcuts import get_object_or_404

from rest_framework.response import Response

from mayan.apps.documents.models import Document
from mayan.apps.rest_api import generics

from .models import DocumentPageContent
from .permissions import permission_content_view
from .serializers import DocumentPageContentSerializer


class APIDocumentPageContentView(generics.RetrieveAPIView):
    """
    Returns the content of the selected document page.
    """
    lookup_url_kwarg = 'page_pk'
    mayan_object_permissions = {
        'GET': (permission_content_view,),
    }
    serializer_class = DocumentPageContentSerializer

    def get_document(self):
        return get_object_or_404(klass=Document, pk=self.kwargs['document_pk'])

    def get_document_version(self):
        return get_object_or_404(
            klass=self.get_document().versions.all(),
            pk=self.kwargs['version_pk']
        )

    def get_queryset(self):
        return self.get_document_version().pages.all()

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()

        try:
            content = instance.content
        except DocumentPageContent.DoesNotExist:
            content = DocumentPageContent.objects.none()

        serializer = self.get_serializer(content)
        return Response(serializer.data)



from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('checkouts', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='documentcheckout',
            name='user',
            field=models.ForeignKey(
                verbose_name='User', blank=True, to=settings.AUTH_USER_MODEL,
                null=True
            ),
            preserve_default=True,
        ),
    ]



from __future__ import absolute_import, unicode_literals

import logging
import os

from django import forms
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.forms import DetailForm

from ..fields import DocumentField
from ..models import Document
from ..literals import DEFAULT_ZIP_FILENAME, PAGE_RANGE_ALL, PAGE_RANGE_CHOICES
from ..utils import get_language, get_language_choices

__all__ = (
    'DocumentDownloadForm', 'DocumentForm', 'DocumentPreviewForm',
    'DocumentPropertiesForm', 'DocumentPrintForm'
)
logger = logging.getLogger(__name__)


class DocumentDownloadForm(forms.Form):
    compressed = forms.BooleanField(
        label=_('Compress'), required=False,
        help_text=_(
            'Download the document in the original format or in a compressed '
            'manner. This option is selectable only when downloading one '
            'document, for multiple documents, the bundle will always be '
            'downloads as a compressed file.'
        )
    )
    zip_filename = forms.CharField(
        initial=DEFAULT_ZIP_FILENAME, label=_('Compressed filename'),
        required=False,
        help_text=_(
            'The filename of the compressed file that will contain the '
            'documents to be downloaded, if the previous option is selected.'
        )
    )

    def __init__(self, *args, **kwargs):
        self.queryset = kwargs.pop('queryset', None)
        super(DocumentDownloadForm, self).__init__(*args, **kwargs)
        if self.queryset.count() > 1:
            self.fields['compressed'].initial = True
            self.fields['compressed'].widget.attrs.update(
                {'disabled': 'disabled'}
            )


class DocumentForm(forms.ModelForm):
    """
    Form sub classes from DocumentForm used only when editing a document
    """
    class Meta:
        fields = ('label', 'description', 'language')
        model = Document
        widgets = {
            'language': forms.Select(
                choices=get_language_choices(), attrs={
                    'class': 'select2'
                }
            )

        }

    def __init__(self, *args, **kwargs):
        document_type = kwargs.pop('document_type', None)

        super(DocumentForm, self).__init__(*args, **kwargs)

        # Is a document (documents app edit) and has been saved (sources
        # app upload)?
        if self.instance and self.instance.pk:
            document_type = self.instance.document_type

        filenames_queryset = document_type.filenames.filter(enabled=True)

        if filenames_queryset:
            self.fields[
                'document_type_available_filenames'
            ] = forms.ModelChoiceField(
                queryset=filenames_queryset,
                required=False,
                label=_('Quick document rename'),
                widget=forms.Select(
                    attrs={
                        'class': 'select2'
                    }
                )
            )
            self.fields['preserve_extension'] = forms.BooleanField(
                label=_('Preserve extension'), required=False,
                help_text=_(
                    'Takes the file extension and moves it to the end of the '
                    'filename allowing operating systems that rely on file '
                    'extensions to open document correctly.'
                )
            )

    def clean(self):
        self.cleaned_data['label'] = self.get_final_label(
            # Fallback to the instance label if there is no label key or
            # there is a label key and is an empty string
            filename=self.cleaned_data.get('label') or self.instance.label
        )

        return self.cleaned_data

    def get_final_label(self, filename):
        if 'document_type_available_filenames' in self.cleaned_data:
            if self.cleaned_data['document_type_available_filenames']:
                if self.cleaned_data['preserve_extension']:
                    filename, extension = os.path.splitext(filename)

                    filename = '{}{}'.format(
                        self.cleaned_data[
                            'document_type_available_filenames'
                        ].filename, extension
                    )
                else:
                    filename = self.cleaned_data[
                        'document_type_available_filenames'
                    ].filename

        return filename


class DocumentPreviewForm(forms.Form):
    def __init__(self, *args, **kwargs):
        document = kwargs.pop('instance', None)
        super(DocumentPreviewForm, self).__init__(*args, **kwargs)
        self.fields['document'].initial = document

    document = DocumentField()


class DocumentPropertiesForm(DetailForm):
    """
    Detail class form to display a document file based properties
    """
    def __init__(self, *args, **kwargs):
        document = kwargs['instance']

        extra_fields = [
            {
                'label': _('Date added'),
                'field': 'date_added',
                'widget': forms.widgets.DateTimeInput
            },
            {'label': _('UUID'), 'field': 'uuid'},
            {
                'label': _('Language'),
                'field': lambda x: get_language(language_code=document.language)
            },
        ]

        if document.latest_version:
            extra_fields += (
                {
                    'label': _('File mimetype'),
                    'field': lambda x: document.file_mimetype or _('None')
                },
                {
                    'label': _('File encoding'),
                    'field': lambda x: document.file_mime_encoding or _(
                        'None'
                    )
                },
                {
                    'label': _('File size'),
                    'field': lambda document: filesizeformat(
                        document.size
                    ) if document.size else '-'
                },
                {'label': _('Exists in storage'), 'field': 'exists'},
                {
                    'label': _('File path in storage'),
                    'field': 'latest_version.file'
                },
                {'label': _('Checksum'), 'field': 'checksum'},
                {'label': _('Pages'), 'field': 'page_count'},
            )

        kwargs['extra_fields'] = extra_fields
        super(DocumentPropertiesForm, self).__init__(*args, **kwargs)

    class Meta:
        fields = ('document_type', 'description')
        model = Document


class DocumentPrintForm(forms.Form):
    page_group = forms.ChoiceField(
        choices=PAGE_RANGE_CHOICES, initial=PAGE_RANGE_ALL,
        widget=forms.RadioSelect
    )
    page_range = forms.CharField(label=_('Page range'), required=False)



from __future__ import unicode_literals


class Endpoint(object):
    def __init__(self, label):
        self.label = label

    @property
    def url(self):
        return '/api/{}/'.format(self.label)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.classes import ErrorLogNamespace

error_log_state_actions = ErrorLogNamespace(
    name='workflow_state_actions', label=_('Workflow state actions')
)



from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0001_initial'),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Source',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'title', models.CharField(
                        max_length=64, verbose_name='Title'
                    )
                ),
                (
                    'enabled', models.BooleanField(
                        default=True, verbose_name='Enabled'
                    )
                ),
            ],
            options={
                'ordering': ('title',),
                'verbose_name': 'Source',
                'verbose_name_plural': 'Sources',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OutOfProcessSource',
            fields=[
                (
                    'source_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.Source'
                    )
                ),
            ],
            options={
                'verbose_name': 'Out of process',
                'verbose_name_plural': 'Out of process',
            },
            bases=('sources.source',),
        ),
        migrations.CreateModel(
            name='IntervalBaseModel',
            fields=[
                (
                    'outofprocesssource_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.OutOfProcessSource'
                    )
                ),
                (
                    'interval', models.PositiveIntegerField(
                        default=600, help_text='Interval in seconds between '
                        'checks for new documents.', verbose_name='Interval'
                    )
                ),
                (
                    'uncompress', models.CharField(
                        help_text='Whether to expand or not, compressed '
                        'archives.', max_length=1, verbose_name='Uncompress',
                        choices=[('y', 'Always'), ('n', 'Never')]
                    )
                ),
            ],
            options={
                'verbose_name': 'Interval source',
                'verbose_name_plural': 'Interval sources',
            },
            bases=('sources.outofprocesssource',),
        ),
        migrations.CreateModel(
            name='EmailBaseModel',
            fields=[
                (
                    'intervalbasemodel_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.IntervalBaseModel'
                    )
                ),
                (
                    'host', models.CharField(
                        max_length=128, verbose_name='Host'
                    )
                ),
                (
                    'ssl', models.BooleanField(
                        default=True, verbose_name='SSL'
                    )
                ),
                (
                    'port', models.PositiveIntegerField(
                        help_text='Typical choices are 110 for POP3, 995 for '
                        'POP3 over SSL, 143 for IMAP, 993 for IMAP over SSL.',
                        null=True, verbose_name='Port', blank=True
                    )
                ),
                (
                    'username', models.CharField(
                        max_length=96, verbose_name='Username'
                    )
                ),
                (
                    'password', models.CharField(
                        max_length=96, verbose_name='Password'
                    )
                ),
            ],
            options={
                'verbose_name': 'Email source',
                'verbose_name_plural': 'Email sources',
            },
            bases=('sources.intervalbasemodel',),
        ),
        migrations.CreateModel(
            name='POP3Email',
            fields=[
                (
                    'emailbasemodel_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.EmailBaseModel'
                    )
                ),
                (
                    'timeout', models.PositiveIntegerField(
                        default=60, verbose_name='Timeout'
                    )
                ),
            ],
            options={
                'verbose_name': 'POP email',
                'verbose_name_plural': 'POP email',
            },
            bases=('sources.emailbasemodel',),
        ),
        migrations.CreateModel(
            name='IMAPEmail',
            fields=[
                (
                    'emailbasemodel_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.EmailBaseModel'
                    )
                ),
                (
                    'mailbox', models.CharField(
                        default='INBOX', help_text='Mail from which to check '
                        'for messages with attached documents.',
                        max_length=64, verbose_name='Mailbox'
                    )
                ),
            ],
            options={
                'verbose_name': 'IMAP email',
                'verbose_name_plural': 'IMAP email',
            },
            bases=('sources.emailbasemodel',),
        ),
        migrations.CreateModel(
            name='InteractiveSource',
            fields=[
                (
                    'source_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.Source'
                    )
                ),
            ],
            options={
                'verbose_name': 'Interactive source',
                'verbose_name_plural': 'Interactive sources',
            },
            bases=('sources.source',),
        ),
        migrations.CreateModel(
            name='SourceTransformation',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'object_id', models.PositiveIntegerField()
                ),
                (
                    'order', models.PositiveIntegerField(
                        default=0, null=True, verbose_name='Order',
                        db_index=True, blank=True
                    )
                ),
                (
                    'transformation', models.CharField(
                        max_length=128, verbose_name='Transformation',
                        choices=[
                            ('resize', 'Resize'), ('rotate', 'Rotate'),
                            ('zoom', 'Zoom')
                        ]
                    )
                ),
                (
                    'arguments', models.TextField(
                        blank=True, help_text="Use dictionaries to indentify "
                        "arguments, example: {'degrees':90}", null=True,
                        verbose_name='Arguments', validators=[]
                    )
                ),
                (
                    'content_type', models.ForeignKey(
                        to='contenttypes.ContentType'
                    )
                ),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'Document source transformation',
                'verbose_name_plural': 'Document source transformations',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='StagingFolderSource',
            fields=[
                (
                    'interactivesource_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.InteractiveSource'
                    )
                ),
                (
                    'folder_path', models.CharField(
                        help_text='Server side filesystem path.',
                        max_length=255, verbose_name='Folder path'
                    )
                ),
                (
                    'preview_width', models.IntegerField(
                        help_text='Width value to be passed to the converter '
                        'backend.', verbose_name='Preview width'
                    )
                ),
                (
                    'preview_height', models.IntegerField(
                        help_text='Height value to be passed to the '
                        'converter backend.', null=True,
                        verbose_name='Preview height', blank=True
                    )
                ),
                (
                    'uncompress', models.CharField(
                        help_text='Whether to expand or not compressed '
                        'archives.', max_length=1, verbose_name='Uncompress',
                        choices=[
                            ('y', 'Always'), ('n', 'Never'), ('a', 'Ask user')
                        ]
                    )
                ),
                (
                    'delete_after_upload', models.BooleanField(
                        default=True, help_text='Delete the file after is '
                        'has been successfully uploaded.',
                        verbose_name='Delete after upload'
                    )
                ),
            ],
            options={
                'verbose_name': 'Staging folder',
                'verbose_name_plural': 'Staging folders',
            },
            bases=('sources.interactivesource',),
        ),
        migrations.CreateModel(
            name='WatchFolderSource',
            fields=[
                (
                    'intervalbasemodel_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.IntervalBaseModel'
                    )
                ),
                (
                    'folder_path', models.CharField(
                        help_text='Server side filesystem path.',
                        max_length=255, verbose_name='Folder path'
                    )
                ),
            ],
            options={
                'verbose_name': 'Watch folder',
                'verbose_name_plural': 'Watch folders',
            },
            bases=('sources.intervalbasemodel',),
        ),
        migrations.CreateModel(
            name='WebFormSource',
            fields=[
                (
                    'interactivesource_ptr', models.OneToOneField(
                        parent_link=True, auto_created=True, primary_key=True,
                        serialize=False, to='sources.InteractiveSource'
                    )
                ),
                (
                    'uncompress', models.CharField(
                        help_text='Whether to expand or not compressed '
                        'archives.', max_length=1, verbose_name='Uncompress',
                        choices=[
                            ('y', 'Always'), ('n', 'Never'), ('a', 'Ask user')
                        ]
                    )
                ),
            ],
            options={
                'verbose_name': 'Web form',
                'verbose_name_plural': 'Web forms',
            },
            bases=('sources.interactivesource',),
        ),
        migrations.AddField(
            model_name='intervalbasemodel',
            name='document_type',
            field=models.ForeignKey(
                verbose_name='Document type', to='documents.DocumentType',
                help_text='Assign a document type to documents uploaded from '
                'this source.'
            ),
            preserve_default=True,
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from mayan.apps.dynamic_search.classes import SearchModel

from .permissions import permission_cabinet_view

cabinet_search = SearchModel(
    app_label='cabinets', model_name='Cabinet',
    permission=permission_cabinet_view,
    serializer_path='mayan.apps.cabinets.serializers.CabinetSerializer'
)

cabinet_search.add_model_field(
    field='label', label=_('Label')
)



#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import mayan

PACKAGE_NAME = 'mayan-edms'
PACKAGE_DIR = 'mayan'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)


def find_packages(directory):
    # Compile the list of packages available, because distutils doesn't have
    # an easy way to do this.
    packages, data_files = [], []
    root_dir = os.path.dirname(__file__)
    if root_dir != '':
        os.chdir(root_dir)

    for dirpath, dirnames, filenames in os.walk(directory):
        if not dirpath.startswith('mayan/media'):
            # Ignore dirnames that start with '.'
            if os.path.basename(dirpath).startswith('.'):
                continue
            if '__init__.py' in filenames:
                packages.append('.'.join(fullsplit(dirpath)))
            elif filenames:
                data_files.append(
                    [dirpath, [os.path.join(dirpath, f) for f in filenames]]
                )

    return packages

install_requires = """
django==1.11.27
Pillow==6.2.2
PyPDF2==1.26.0
PyYAML==5.1.2
celery==4.3.0
django-activity-stream==0.7.0
django-celery-beat==1.5.0
django-colorful==1.3
django-cors-headers==2.5.2
django-formtools==2.1
django-mathfilters==0.4.0
django-model-utils==3.1.2
django-mptt==0.9.1
django-pure-pagination==0.3.0
django-qsstats-magic==1.0.0
django-solo==1.1.3
django-stronghold==0.3.0
django-timezone-field==3.1
django-widget-tweaks==1.4.5
djangorestframework==3.7.7
djangorestframework-recursive==0.1.2
drf-yasg==1.6.0
flanker==0.9.0
flex==6.14.0
furl==2.0.0
fusepy==3.0.1
gevent==1.4.0
graphviz==0.10.1
gunicorn==19.9.0
kombu==4.6.7
mock==2.0.0
node-semver==0.6.1
pathlib2==2.3.5
pycountry==18.12.8
pyocr==0.6
python-dateutil==2.8.0
python-magic==0.4.15
python_gnupg==0.3.9
pytz==2019.1
requests==2.21.0
sh==1.12.14
swagger-spec-validator==2.4.3
whitenoise==4.1.4
""".split()

with open('README.rst') as f:
    readme = f.read()

with open('HISTORY.rst') as f:
    history = f.read()

setup(
    author='Roberto Rosario',
    author_email='roberto.rosario@mayan-edms.com',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Education',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Communications :: File Sharing',
    ],
    description=mayan.__description__,
    include_package_data=True,
    install_requires=install_requires,
    license='Apache 2.0',
    long_description=readme + '\n\n' + history,
    name=PACKAGE_NAME,
    packages=find_packages(PACKAGE_DIR),
    platforms=['any'],
    python_requires = '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    scripts=['mayan/bin/mayan-edms.py'],
    url='https://gitlab.com/mayan-edms/mayan-edms',
    version=mayan.__version__,
    zip_safe=False,
)



__all__ = (
    'GPGException', 'VerificationError', 'SigningError',
    'DecryptionError', 'KeyDeleteError', 'KeyGenerationError',
    'KeyFetchingError', 'KeyDoesNotExist', 'KeyImportError'
)


class GPGException(Exception):
    pass


class VerificationError(GPGException):
    pass


class SigningError(GPGException):
    pass


class DecryptionError(GPGException):
    pass


class KeyDeleteError(GPGException):
    pass


class KeyGenerationError(GPGException):
    pass


class KeyFetchingError(GPGException):
    """
    Unable to receive key or key not found
    """


class KeyDoesNotExist(GPGException):
    pass


class KeyImportError(GPGException):
    pass


class NeedPassphrase(GPGException):
    """
    Passphrase is needed but none was provided
    """


class PassphraseError(GPGException):
    """
    Passphrase provided is incorrect
    """



from __future__ import absolute_import, unicode_literals

from datetime import timedelta

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.queues import queue_tools
from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_fast, worker_medium

from .literals import (
    CHECK_DELETE_PERIOD_INTERVAL, CHECK_TRASH_PERIOD_INTERVAL,
    DELETE_STALE_STUBS_INTERVAL
)

queue_converter = CeleryQueue(
    name='converter', label=_('Converter'), transient=True, worker=worker_fast
)
queue_documents_periodic = CeleryQueue(
    name='documents_periodic', label=_('Documents periodic'), transient=True, worker=worker_medium
)
queue_uploads = CeleryQueue(
    name='uploads', label=_('Uploads'), worker=worker_medium
)
queue_documents = CeleryQueue(
    name='documents', label=_('Documents'), worker=worker_medium
)

queue_converter.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_generate_document_page_image',
    label=_('Generate document page image')
)

queue_documents.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_delete_document',
    label=_('Delete a document')
)
queue_documents.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_clean_empty_duplicate_lists',
    label=_('Clean empty duplicate lists')
)

queue_documents_periodic.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_check_delete_periods',
    label=_('Check document type delete periods'),
    name='task_check_delete_periods',
    schedule=timedelta(
        seconds=CHECK_DELETE_PERIOD_INTERVAL
    ),
)
queue_documents_periodic.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_check_trash_periods',
    label=_('Check document type trash periods'),
    name='task_check_trash_periods',
    schedule=timedelta(seconds=CHECK_TRASH_PERIOD_INTERVAL),
)
queue_documents_periodic.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_delete_stubs',
    label=_('Delete document stubs'),
    name='task_delete_stubs',
    schedule=timedelta(seconds=DELETE_STALE_STUBS_INTERVAL),
)

queue_tools.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_scan_duplicates_all',
    label=_('Duplicated document scan')
)

queue_uploads.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_update_page_count',
    label=_('Update document page count')
)
queue_uploads.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_upload_new_version',
    label=_('Upload new document version')
)
queue_uploads.add_task_type(
    dotted_path='mayan.apps.documents.tasks.task_scan_duplicates_for',
    label=_('Scan document duplicates')
)



from __future__ import unicode_literals

from django.db import migrations
from django.conf import settings


def operation_migrate_old_comments(apps, schema_editor):
    # https://code.djangoproject.com/ticket/24282
    # If someone has a better solution until Django 1.8, would appreciate
    # a pull-request :)

    try:
        from django.contrib.comments.models import Comment as OldComment
    except ImportError:
        # Django > 1.7
        pass
    else:

        Comment = apps.get_model(
            app_label='document_comments', model_name='Comment'
        )
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )
        User = apps.get_model(*settings.AUTH_USER_MODEL.split('.'))

        for old_comment in OldComment.objects.using(schema_editor.connection.alias).all():
            comment = Comment(
                document=Document.objects.using(
                    schema_editor.connection.alias
                ).get(pk=old_comment.object_pk),
                user=User(old_comment.user.pk),
                comment=old_comment.comment,
                submit_date=old_comment.submit_date,
            )
            comment.save()


class Migration(migrations.Migration):

    dependencies = [
        ('document_comments', '0002_auto_20150729_2144'),
        ('documents', '0001_initial'),
        ('sites', '0001_initial'),
        ('contenttypes', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL)
    ]

    operations = [
        migrations.RunPython(code=operation_migrate_old_comments),
    ]



from __future__ import absolute_import, unicode_literals

from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..models import SmartLink
from ..permissions import (
    permission_smart_link_create, permission_smart_link_delete,
    permission_smart_link_edit, permission_smart_link_view
)

from .literals import (
    TEST_SMART_LINK_DYNAMIC_LABEL, TEST_SMART_LINK_LABEL_EDITED,
    TEST_SMART_LINK_LABEL
)
from .mixins import (
    SmartLinkDocumentViewTestMixin, SmartLinkTestMixin,
    SmartLinkViewTestMixin
)


class SmartLinkViewTestCase(
    SmartLinkTestMixin, SmartLinkViewTestMixin, GenericViewTestCase
):
    def test_smart_link_create_view_no_permission(self):
        response = self._request_test_smart_link_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(SmartLink.objects.count(), 0)

    def test_smart_link_create_view_with_permission(self):
        self.grant_permission(permission=permission_smart_link_create)

        response = self._request_test_smart_link_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(SmartLink.objects.count(), 1)
        self.assertEqual(
            SmartLink.objects.first().label, TEST_SMART_LINK_LABEL
        )

    def test_smart_link_delete_view_no_permission(self):
        self._create_test_smart_link()

        response = self._request_test_smart_link_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(SmartLink.objects.count(), 1)

    def test_smart_link_delete_view_with_access(self):
        self._create_test_smart_link()
        self.grant_access(
            obj=self.test_smart_link, permission=permission_smart_link_delete
        )

        response = self._request_test_smart_link_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(SmartLink.objects.count(), 0)

    def test_smart_link_edit_view_no_permission(self):
        self._create_test_smart_link()

        response = self._request_test_smart_link_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_smart_link.refresh_from_db()
        self.assertEqual(self.test_smart_link.label, TEST_SMART_LINK_LABEL)

    def test_smart_link_edit_view_with_access(self):
        self._create_test_smart_link()

        self.grant_access(
            obj=self.test_smart_link, permission=permission_smart_link_edit
        )

        response = self._request_test_smart_link_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_smart_link.refresh_from_db()
        self.assertEqual(
            self.test_smart_link.label, TEST_SMART_LINK_LABEL_EDITED
        )


class SmartLinkDocumentViewTestCase(
    SmartLinkTestMixin, SmartLinkDocumentViewTestMixin,
    GenericDocumentViewTestCase
):
    def setUp(self):
        super(SmartLinkDocumentViewTestCase, self).setUp()
        self._create_test_smart_link()
        self.test_smart_link.document_types.add(self.test_document_type)

        self.test_smart_link_2 = SmartLink.objects.create(
            label=TEST_SMART_LINK_LABEL,
            dynamic_label=TEST_SMART_LINK_DYNAMIC_LABEL
        )
        self.test_smart_link_2.document_types.add(self.test_document_type)

    def test_document_smart_link_list_view_no_permission(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_smart_link_document_instances_view()
        # Text must appear 3 times, two for the title and one for the template
        # heading. The two smart links are not shown.
        self.assertContains(
            response=response, text=self.test_document.label, count=3,
            status_code=200
        )

    def test_document_smart_link_list_view_with_permission(self):
        self.grant_access(
            obj=self.test_smart_link, permission=permission_smart_link_view
        )
        self.grant_access(
            obj=self.test_smart_link_2, permission=permission_smart_link_view
        )
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        response = self._request_test_smart_link_document_instances_view()
        # Text must appear 5 times: 3 for the windows title and template
        # heading, plus 2 for the test.
        self.assertContains(
            response, text=self.test_document.label, count=5, status_code=200
        )



from __future__ import unicode_literals

from mayan.apps.common.tests.base import GenericViewTestCase
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..models import WorkflowTransition
from ..permissions import (
    permission_workflow_edit, permission_workflow_view,
    permission_workflow_transition
)

from .literals import (
    TEST_WORKFLOW_TRANSITION_FIELD_HELP_TEXT,
    TEST_WORKFLOW_TRANSITION_FIELD_LABEL, TEST_WORKFLOW_TRANSITION_FIELD_NAME,
    TEST_WORKFLOW_TRANSITION_FIELD_TYPE, TEST_WORKFLOW_TRANSITION_LABEL,
    TEST_WORKFLOW_TRANSITION_LABEL_EDITED
)
from .mixins import (
    WorkflowTestMixin, WorkflowViewTestMixin, WorkflowTransitionViewTestMixin
)


class WorkflowTransitionViewTestCase(
    WorkflowTestMixin, WorkflowViewTestMixin, WorkflowTransitionViewTestMixin,
    GenericViewTestCase
):
    def test_create_test_workflow_transition_no_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        response = self._request_test_workflow_transition_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(WorkflowTransition.objects.count(), 0)

    def test_create_test_workflow_transition_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_transition_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(WorkflowTransition.objects.count(), 1)
        self.assertEqual(
            WorkflowTransition.objects.all()[0].label,
            TEST_WORKFLOW_TRANSITION_LABEL
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].origin_state,
            self.test_workflow_state_1
        )
        self.assertEqual(
            WorkflowTransition.objects.all()[0].destination_state,
            self.test_workflow_state_2
        )

    def test_delete_workflow_transition_no_permissions(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        response = self._request_test_workflow_transition_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertTrue(
            self.test_workflow_transition in WorkflowTransition.objects.all()
        )

    def test_delete_workflow_transition_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.grant_access(permission=permission_workflow_edit, obj=self.test_workflow)

        response = self._request_test_workflow_transition_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertFalse(
            self.test_workflow_transition in WorkflowTransition.objects.all()
        )

    def test_edit_workflow_transition_no_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        response = self._request_test_workflow_transition_edit_view()
        self.assertEqual(response.status_code, 404)

        self.test_workflow_transition.refresh_from_db()
        self.assertEqual(
            self.test_workflow_transition.label, TEST_WORKFLOW_TRANSITION_LABEL
        )

    def test_edit_workflow_transition_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_transition_edit_view()
        self.assertEqual(response.status_code, 302)

        self.test_workflow_transition.refresh_from_db()
        self.assertEqual(
            self.test_workflow_transition.label,
            TEST_WORKFLOW_TRANSITION_LABEL_EDITED
        )

    def test_workflow_transition_list_no_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        response = self._request_test_workflow_transition_list_view()
        self.assertNotContains(
            response=response, text=self.test_workflow_transition.label,
            status_code=404
        )

    def test_workflow_transition_list_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_view
        )

        response = self._request_test_workflow_transition_list_view()
        self.assertContains(
            response=response, text=self.test_workflow_transition.label,
            status_code=200
        )


class WorkflowTransitionDocumentViewTestCase(
    WorkflowTestMixin, WorkflowViewTestMixin, WorkflowTransitionViewTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_document = False

    def setUp(self):
        super(WorkflowTransitionDocumentViewTestCase, self).setUp()
        self._create_test_workflow()
        self.test_workflow.document_types.add(self.test_document_type)
        self._create_test_workflow_states()
        self._create_test_workflow_transitions()
        self.upload_document()
        self.test_workflow_instance = self.test_document.workflows.first()

    def test_transition_workflow_no_access(self):
        """
        Test transitioning a workflow without the transition workflow
        permission.
        """
        response = self._request_test_workflow_transition()
        self.assertEqual(response.status_code, 404)

        # Workflow should remain in the same initial state
        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_1
        )

    def test_transition_workflow_with_workflow_access(self):
        """
        Test transitioning a workflow by granting the transition workflow
        permission to the role.
        """
        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_transition
        )

        response = self._request_test_workflow_transition()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_2
        )

    def test_transition_workflow_with_transition_access(self):
        """
        Test transitioning a workflow by granting the transition workflow
        permission to the role.
        """
        self.grant_access(
            obj=self.test_workflow_transition,
            permission=permission_workflow_transition
        )

        response = self._request_test_workflow_transition()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_instance.get_current_state(),
            self.test_workflow_state_2
        )


class WorkflowTransitionEventViewTestCase(
    WorkflowTestMixin, GenericDocumentViewTestCase
):
    def _request_test_workflow_transition_event_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_transition_events',
            kwargs={'pk': self.test_workflow_transition.pk}
        )

    def test_workflow_transition_event_list_no_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        response = self._request_test_workflow_transition_event_list_view()
        self.assertEqual(response.status_code, 404)

    def test_workflow_transition_event_list_with_access(self):
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_transition_event_list_view()
        self.assertEqual(response.status_code, 200)


class WorkflowTransitionFieldViewTestCase(
    WorkflowTestMixin, WorkflowTransitionViewTestMixin, GenericViewTestCase
):
    def setUp(self):
        super(WorkflowTransitionFieldViewTestCase, self).setUp()
        self._create_test_workflow()
        self._create_test_workflow_states()
        self._create_test_workflow_transition()

    def _create_test_workflow_transition_field(self):
        self.test_workflow_transition_field = self.test_workflow_transition.fields.create(
            field_type=TEST_WORKFLOW_TRANSITION_FIELD_TYPE,
            name=TEST_WORKFLOW_TRANSITION_FIELD_NAME,
            label=TEST_WORKFLOW_TRANSITION_FIELD_LABEL,
            help_text=TEST_WORKFLOW_TRANSITION_FIELD_HELP_TEXT
        )

    def _request_test_workflow_transition_field_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_transition_field_list',
            kwargs={'pk': self.test_workflow_transition.pk}
        )

    def test_workflow_transition_field_list_view_no_permission(self):
        self._create_test_workflow_transition_field()

        response = self._request_test_workflow_transition_field_list_view()
        self.assertNotContains(
            response=response,
            text=self.test_workflow_transition_field.label,
            status_code=404
        )

    def test_workflow_transition_field_list_view_with_access(self):
        self._create_test_workflow_transition_field()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_test_workflow_transition_field_list_view()
        self.assertContains(
            response=response,
            text=self.test_workflow_transition_field.label,
            status_code=200
        )

    def _request_workflow_transition_field_create_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_field_create',
            kwargs={'pk': self.test_workflow_transition.pk},
            data={
                'field_type': TEST_WORKFLOW_TRANSITION_FIELD_TYPE,
                'name': TEST_WORKFLOW_TRANSITION_FIELD_NAME,
                'label': TEST_WORKFLOW_TRANSITION_FIELD_LABEL,
                'help_text': TEST_WORKFLOW_TRANSITION_FIELD_HELP_TEXT
            }
        )

    def test_workflow_transition_field_create_view_no_permission(self):
        workflow_transition_field_count = self.test_workflow_transition.fields.count()

        response = self._request_workflow_transition_field_create_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count
        )

    def test_workflow_transition_field_create_view_with_access(self):
        workflow_transition_field_count = self.test_workflow_transition.fields.count()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_workflow_transition_field_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count + 1
        )

    def _request_workflow_transition_field_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_transition_field_delete',
            kwargs={'pk': self.test_workflow_transition_field.pk},
        )

    def test_workflow_transition_field_delete_view_no_permission(self):
        self._create_test_workflow_transition_field()

        workflow_transition_field_count = self.test_workflow_transition.fields.count()

        response = self._request_workflow_transition_field_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count
        )

    def test_workflow_transition_field_delete_view_with_access(self):
        self._create_test_workflow_transition_field()

        workflow_transition_field_count = self.test_workflow_transition.fields.count()

        self.grant_access(
            obj=self.test_workflow, permission=permission_workflow_edit
        )

        response = self._request_workflow_transition_field_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self.test_workflow_transition.fields.count(),
            workflow_transition_field_count - 1
        )



from __future__ import unicode_literals

from django.contrib import admin

from .models import EventSubscription, Notification, StoredEventType


@admin.register(EventSubscription)
class EventSubscriptionAdmin(admin.ModelAdmin):
    list_display = ('user', 'stored_event_type')


@admin.register(StoredEventType)
class StoredEventTypeAdmin(admin.ModelAdmin):
    readonly_fields = ('name', '__str__')


@admin.register(Notification)
class NotificationAdmin(admin.ModelAdmin):
    list_display = ('user', 'action', 'read')



from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_cabinet = Icon(driver_name='fontawesome', symbol='columns')
icon_cabinet_add = Icon(driver_name='fontawesome', symbol='plus')
icon_cabinet_child_add = Icon(driver_name='fontawesome', symbol='plus')
icon_cabinet_create = Icon(driver_name='fontawesome', symbol='plus')

icon_cabinet_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='columns',
    secondary_symbol='plus'
)

icon_cabinet_delete = Icon(driver_name='fontawesome', symbol='times')
icon_cabinet_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_cabinet_list = Icon(driver_name='fontawesome', symbol='columns')
icon_cabinet_view = Icon(driver_name='fontawesome', symbol='columns')

icon_document_cabinet_add = Icon(
    driver_name='fontawesome-dual', primary_symbol='columns',
    secondary_symbol='arrow-right'
)
icon_document_cabinet_list = icon_cabinet_list
icon_document_cabinet_remove = Icon(
    driver_name='fontawesome-dual', primary_symbol='columns',
    secondary_symbol='minus'
)
icon_document_multiple_cabinet_add = Icon(
    driver_name='fontawesome-dual', primary_symbol='columns',
    secondary_symbol='arrow-right'
)
icon_document_multiple_cabinet_remove = Icon(
    driver_name='fontawesome-dual', primary_symbol='columns',
    secondary_symbol='minus'
)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0015_auto_20170206_0835'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sanescanner',
            name='resolution',
            field=models.PositiveIntegerField(
                blank=True,
                help_text='Sets the resolution of the scanned image in '
                'DPI (dots per inch). Typical value is 200. If this option '
                'is not supported by your scanner, leave it blank.',
                null=True, verbose_name='Resolution'
            ),
        ),
    ]



from __future__ import unicode_literals

from mayan.apps.documents.models import Document
from mayan.apps.documents.permissions import permission_document_create
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase
from mayan.apps.sources.models import WebFormSource
from mayan.apps.sources.tests.literals import (
    TEST_SOURCE_LABEL, TEST_SOURCE_UNCOMPRESS_N
)

from .mixins import TagTestMixin, TaggedDocumentUploadViewTestMixin


class TaggedDocumentUploadViewTestCase(
    TaggedDocumentUploadViewTestMixin, TagTestMixin,
    GenericDocumentViewTestCase
):
    auto_upload_document = False

    def setUp(self):
        super(TaggedDocumentUploadViewTestCase, self).setUp()
        self.test_source = WebFormSource.objects.create(
            enabled=True, label=TEST_SOURCE_LABEL,
            uncompress=TEST_SOURCE_UNCOMPRESS_N
        )

    def test_upload_interactive_view_with_access(self):
        self._create_test_tag()

        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )
        response = self._request_upload_interactive_document_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(self.test_tag in Document.objects.first().tags.all())

    def test_upload_interactive_multiple_tags_view_with_access(self):
        self._create_test_tag()
        self._create_test_tag_2()

        self.grant_access(
            obj=self.test_document_type, permission=permission_document_create
        )
        response = self._request_upload_interactive_document_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertTrue(self.test_tag in Document.objects.first().tags.all())
        self.assertTrue(self.test_tag_2 in Document.objects.first().tags.all())



from __future__ import absolute_import, unicode_literals

from datetime import timedelta

from django.utils.translation import ugettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_medium, worker_slow

from .literals import DELETE_STALE_UPLOADS_INTERVAL

queue_default = CeleryQueue(
    default_queue=True, label=_('Default'), name='default', worker=worker_medium
)
queue_tools = CeleryQueue(label=_('Tools'), name='tools', worker=worker_slow)
queue_common_periodic = CeleryQueue(
    label=_('Common periodic'), name='common_periodic', transient=True,
    worker=worker_slow
)
queue_common_periodic.add_task_type(
    dotted_path='mayan.apps.common.tasks.task_delete_stale_uploads',
    label=_('Delete stale uploads'), name='task_delete_stale_uploads',
    schedule=timedelta(
        seconds=DELETE_STALE_UPLOADS_INTERVAL
    )
)



from __future__ import unicode_literals

TEST_CASE_GROUP_NAME = 'test case group'

TEST_CASE_SUPERUSER_EMAIL = 'test_case_superuser@example.com'
TEST_CASE_SUPERUSER_PASSWORD = 'test case superuser password'
TEST_CASE_SUPERUSER_USERNAME = 'test_case_superuser'

TEST_CASE_USER_EMAIL = 'test_case_user@example.com'
TEST_CASE_USER_PASSWORD = 'test case user password'
TEST_CASE_USER_USERNAME = 'test_case_user'

TEST_GROUP_NAME = 'test group'
TEST_GROUP_NAME_EDITED = 'test group edited'
TEST_GROUP_2_NAME = 'test group 2'
TEST_GROUP_2_NAME_EDITED = 'test group 2 edited'

TEST_USER_EMAIL = 'testuser@example.com'
TEST_USER_PASSWORD = 'test user password'
TEST_USER_PASSWORD_EDITED = 'test user password edited'
TEST_USER_USERNAME = 'test_user'
TEST_USER_USERNAME_EDITED = 'test_user_edited'

TEST_USER_2_EMAIL = 'testuser2@example.com'
TEST_USER_2_PASSWORD = 'test user 2 password'
TEST_USER_2_PASSWORD_EDITED = 'test user 2 password edited'
TEST_USER_2_USERNAME = 'test_user_2'
TEST_USER_2_USERNAME_EDITED = 'test_user_2_edited'



from __future__ import unicode_literals

from mayan.apps.storage.utils import get_storage_subclass

from .settings import (
    setting_workflowimagecache_storage,
    setting_workflowimagecache_storage_arguments
)

storage_workflowimagecache = get_storage_subclass(
    dotted_path=setting_workflowimagecache_storage.value
)(**setting_workflowimagecache_storage_arguments.value)



from __future__ import unicode_literals

import logging

from django.apps import apps

from mayan.celery import app

logger = logging.getLogger(__name__)


@app.task(ignore_result=True)
def task_parse_document_version(document_version_pk):
    DocumentVersion = apps.get_model(
        app_label='documents', model_name='DocumentVersion'
    )
    DocumentPageContent = apps.get_model(
        app_label='document_parsing', model_name='DocumentPageContent'
    )

    document_version = DocumentVersion.objects.get(
        pk=document_version_pk
    )
    logger.info(
        'Starting parsing for document version: %s', document_version
    )
    DocumentPageContent.objects.process_document_version(
        document_version=document_version
    )



from __future__ import unicode_literals

from django.utils.six.moves import xmlrpc_client
from django.utils.translation import ugettext_lazy as _

import mayan

from .exceptions import DependenciesException
from .literals import MAYAN_PYPI_NAME, PYPI_URL

MESSAGE_NOT_LATEST = '''The version you are using is outdated. The latest
version is %s'''
MESSAGE_UNKNOWN_VERSION = '''It is not possible to determine the latest
version available.'''
MESSAGE_UNEXPECTED_ERROR = '''Unexpected error trying to determine the
latest version available. Make sure your installation has a connection to
the internet; %s'''
MESSAGE_UP_TO_DATE = 'Your version is up-to-date.'


class PyPIClient(object):
    class NotLatestVersion(DependenciesException):
        """
        The installed version is not the latest available version
        """
        def __init__(self, upstream_version):
            self.upstream_version = upstream_version

    class UnknownLatestVersion(DependenciesException):
        """
        It is not possible to determine what is the latest upstream version.
        """

    def check_version(self):
        versions = self.get_versions()
        if not versions:
            raise PyPIClient.UnknownLatestVersion
        else:
            if versions[0] != mayan.__version__:
                raise PyPIClient.NotLatestVersion(
                    upstream_version=versions[0]
                )

    def check_version_verbose(self):
        try:
            self.check_version()
        except PyPIClient.NotLatestVersion as exception:
            message = _(MESSAGE_NOT_LATEST) % exception.upstream_version
        except PyPIClient.UnknownLatestVersion:
            message = _(MESSAGE_UNKNOWN_VERSION)
        except Exception as exception:
            message = _(MESSAGE_UNEXPECTED_ERROR) % exception
        else:
            message = _(MESSAGE_UP_TO_DATE)

        return message

    def get_server_proxy(self):
        return xmlrpc_client.ServerProxy(PYPI_URL)

    def get_versions(self):
        server_proxy = self.get_server_proxy()
        return server_proxy.package_releases(MAYAN_PYPI_NAME)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('file_caching', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cache',
            name='label',
            field=models.CharField(
                help_text='A short text describing the cache.',
                max_length=128, verbose_name='Label'
            ),
        ),
        migrations.AlterField(
            model_name='cache',
            name='maximum_size',
            field=models.PositiveIntegerField(
                help_text='Maximum size of the cache in bytes.',
                verbose_name='Maximum size'
            ),
        ),
        migrations.AlterField(
            model_name='cache',
            name='name',
            field=models.CharField(
                help_text='Internal name of the cache.', max_length=128,
                unique=True, verbose_name='Name'
            ),
        ),
        migrations.AlterField(
            model_name='cache',
            name='storage_instance_path',
            field=models.CharField(
                help_text='Dotted path to the actual storage class used '
                'for the cache.', max_length=255, unique=True,
                verbose_name='Storage instance path'
            ),
        ),
    ]
